# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Details;
# File: VMware/Management/Details.pm

use strict;

use VMware::Control '1.01';
use VMware::Control::Server ();
use VMware::Control::VM ();
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials
                                 append_error
                                 append_vm
                                 append_user
                                 append_build
                                 append_host
                                 append_agent
                                 append_product
                                 connect_ctrl_obj );

use XML::DOM ();
use XML::Sablotron ();
use CGI ();

my $doc;
my $q;
my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();
  $q = new CGI( shift( ) );
  my %credentials = hash_credentials( );
  my $parser = new XML::DOM::Parser;
  $doc = $parser->parse( "<server></server>" );
  my $root = $doc->getDocumentElement( );
  my $version = VMware::Control::version( );
  my $want_version = "1.01";
  my $processor = new XML::Sablotron();
  my $xsl = $ENV{ "style_DEFAULT" };
  my ( $server, $server_ok, $vm, $err, $errstr );
  my @vms;

  append_error( $doc, $root, undef, "Virtual machine control API version mismatch. Want: $want_version. Have: $version." )
    unless $want_version eq $version;

  $server = &VMware::Control::Server::new( undef, 0, $credentials{ "l" }, $credentials{ "m" } );
  ( $server_ok, $err, $errstr ) = connect_ctrl_obj( $server );
  if ( $server_ok ) {
    $vm = $q->param( "c" );
    $vm ||= $q->param( "vm" );
    if ( !$vm ) {
      @vms = $server->enumerate( );
      $vm = $vms[ 0 ];
    }
  } else {
    # [ ! ] XXX Return an error about the inability to connect to vmauthd 
    trace( "$err: $errstr", "error" );
    append_error( $doc, $root, $err, $errstr );
  }

  if ( $vm ) {
    append_host( $doc, $root );
    append_agent( $doc, $root );
    append_user( $doc, $root, $credentials{ "l" } );
    append_build( $doc, $root );
    append_vm($doc, $root, $vm, $server);
  } else {
    trace( "There do not appear to be any virtual machines.", "error" );
  }
  append_product( $doc, $root, $server );

  if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
    my $xmlfile = &VMware::Management::Util::tmpdir() . "details.xml";
    string_to_file( $doc->toString( ), $xmlfile, 1 );
  }

  if ( $server_ok ) {
    $server->disconnect( );
    trace( "\$server->disconnect( ) called ($server).", "notice" );
  }

  $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
  $p->start("XML::Sablotron::RunProcessor");
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template", file_to_string( $xsl ), "data", $doc->toString( ) ] );
  $p->end();

  $doc->dispose( );
  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

1;
