# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Dialog;
# File: VMware/Management/Dialog.pm

use strict;
use VMware::Control::Profiler;
use XML::DOM;

#    $_p ... parser instance
#    $_d ... document object
# $_root ... document element
# $_head ... head element
# $_body ... body element
my ( $_p, $_d, $_root, $_head, $_body );
my $p = VMware::Control::Profiler::getInstance("mui");

sub new {
  my $that = shift;
  my $this = {};
  my $class = ref( $that ) || $that;
  bless( $this, $class );
  $this->_initialize( @_ );
  return $this;
}

sub _initialize {
    my $stub = $p->profile();
  my ( $this, $type ) = @_;

  $_p = new XML::DOM::Parser;
  $_d = $_p->parse( "<dialog></dialog>" );
  $_root = $_d->getDocumentElement( );
  $_head = $_root->appendChild( $_d->createElement( "head" ) );
  $_body = $_root->appendChild( $_d->createElement( "body" ) );
  $this->set_type( $type ) unless !$type;

}

sub set_type {
  my ( $this, $type ) = @_;
  
  if ( $type ) {
    $_root->setAttribute( "type", $type );
  }
}

sub set_title {
  my ( $this, $title ) = @_;
  my @nodelist = $_head->getElementsByTagName( "title" );
  my $node = $_d->createElement( "title" );
  $node->appendChild( $_d->createTextNode( $title ) );

  foreach my $t ( @nodelist ) {
    if ( $t->getParentNode( ) == $_head ) {
      $_head->removeChild( $t );
    }
  }
  
  $_head->appendChild( $node );
}

sub set_vm {
    my $stub = $p->profile();
  my ( $this ) = @_;
  my @nodelist = $_head->getElementsByTagName( "vm" );
  my $node = $_d->createElement( "vm" );

  foreach my $t ( @nodelist ) {
    if ( $t->getParentNode( ) == $_head ) {
      $_head->removeChild( $t );
    }
  }
  
  $_head->appendChild( $node );
}

sub set_vm_cfg {
    my $stub = $p->profile();
  my ( $this, $parent, $cfg, $display_name ) = @_;
  my @nodelist = $parent->getElementsByTagName( "cfg" );
  my $node = $_d->createElement( "cfg" );
  $node->setAttribute( "displayName", $display_name ? $display_name : $cfg );
  $node->appendChild( $_d->createTextNode( $cfg ) );

  foreach my $t ( @nodelist ) {
    if ( $t->getParentNode( ) == $parent ) {
      $parent->removeChild( $t );
    }
  }
  
  $parent->appendChild( $node );
}

sub set_operation {
    my $stub = $p->profile();
  my ( $this, $operation ) = @_;
  my @nodelist = $_head->getElementsByTagName( "operation" );
  my $node = $_d->createElement( "operation" );
  $node->appendChild( $_d->createTextNode( $operation ) );

  foreach my $t ( @nodelist ) {
    if ( $t->getParentNode( ) == $_head ) {
      $_head->removeChild( $t );
    }
  }
  
  $_head->appendChild( $node );
}

# &! This needs to be factored better. Requires an operation parent.
sub set_operation_status {
  my ( $this, $parent, $status ) = @_;
  $parent->setAttribute( "status", $status );
}

sub set_description {
  my $this = shift;
  my @description = split /\n\n+/, shift;
  my @nodelist = $_body->getElementsByTagName( "p" );

  foreach my $p ( @nodelist ) {
    if ( $p->getParentNode( ) == $_body ) {
      $_body->removeChild( $p );
    }
  }
  
  foreach my $p ( @description ) {
    my $node = $_body->appendChild( $_d->createElement( "p" ) );
    $node->appendChild( $_d->createTextNode( $p ) );
  }
}

sub get_action_attr {
  my ( $this, $name, $attr ) = @_;
  
  _get_named_node_attr( $_body, "action", $name, $attr );
}

sub set_action_attr {
  my ( $this, $name, $attr, $valu ) = @_;

  _set_named_node_attr( $_body, "action", $name, $attr, $valu );
}

sub append_action {
  my ( $this, $name, $label, $valu, $args ) = @_;

  my $node = $_body->appendChild( $_d->createElement( "action" ) );
  $node->setAttribute( "name", $name );
  $node->setAttribute( "label", $label );
  $node->setAttribute( "value", $valu );
  $node->setAttribute( "args", $args );  

  return $node;
}

sub remove_action {
  my ( $this, $name ) = @_;

  _remove_named_node( $_body, "action", $name );
}

sub get_args_attr {
  my ( $this, $name, $attr ) = @_;
  
  _get_named_node_attr( $_body, "args", $name, $attr );
}

sub set_args_attr {
  my ( $this, $name, $attr, $valu ) = @_;

  _set_named_node_attr( $_body, "args", $name, $attr, $valu );
}

sub append_args {
  my ( $this, $name ) = @_;

  my $node = $_body->appendChild( $_d->createElement( "args" ) );
  $node->setAttribute( "name", $name );

  return $node;
}

sub get_args {
  my ( $this, $name ) = @_;

  return _getNamedNode( $_body, "action", $name );
}

sub remove_args {
  my ( $this, $name ) = @_;

  _remove_named_node( $_body, "args", $name );
}

sub get_select_attr {
  my ( $this, $parent, $name, $attr ) = @_;
  
  _get_named_node_attr( $parent, "select", $name, $attr );
}

sub set_select_attr {
  my ( $this, $parent, $name, $attr ) = @_;
  
  _get_named_node_attr( $parent, "select", $name, $attr );
}

sub append_select {
  my ( $this, $parent, $name, $type ) = @_;

  my $node = $parent->appendChild( $_d->createElement( "select" ) );
  $node->setAttribute( "name", $name );
  $node->setAttribute( "type", $type );

  return $node;
}

sub remove_select {
  my ( $this, $parent, $name ) = @_;

  _remove_named_node( $parent, "select", $name );  
}

sub get_option_attr {
  my ( $this, $parent, $name, $attr ) = @_;
  
  _get_named_node_attr( $parent, "option", $name, $attr );
}

sub set_option_attr {
  my ( $this, $parent, $name, $attr ) = @_;
  
  _get_named_node_attr( $parent, "option", $name, $attr );
}

sub append_option {
  my ( $this, $parent, $name, $valu ) = @_;

  my $node = $parent->appendChild( $_d->createElement( "option" ) );
  $node->setAttribute( "name", $name );
  $node->setAttribute( "value", $valu );

  return $node;
}

sub remove_option {
  my ( $this, $parent, $name ) = @_;

  _remove_named_node( $parent, "option", $name );  
}

sub append_range {
  my ( $this, $parent, $name, $valu, $min, $max, $step ) = @_;

  my $node = $parent->appendChild( $_d->createElement( "range" ) );
  $node->setAttribute( "name", $name );
  $node->setAttribute( "value", $valu );
  $node->setAttribute( "min", $min );
  $node->setAttribute( "max", $max );
  $node->setAttribute( "step", $step );

  return $node;
}

sub append_input {
  my ( $this, $parent, $name, $valu, $hold ) = @_;

  my $node = $parent->appendChild( $_d->createElement( "input" ) );
  $node->setAttribute( "name", $name );
  $node->setAttribute( "value", $valu );
  $node->setAttribute( "hold", $hold );

  return $node;
}

sub append_multipartKey {
  my ( $this, $name, $key ) = @_;

  my $node = $_head->appendChild( $_d->createElement( "multipartKey" ) );
  $node->setAttribute( "name", $name );
  $node->appendChild( $_d->createTextNode( $key ) );

  return $node;
}

sub to_string {
  return $_d->toString( );
}

sub _get_named_node {
  # match name_attr attribute if supplied.
  my ( $parent, $tag, $name, $name_attr ) = @_;
  $name_attr ||= "name";
  # otherwise use the name attribute.
  my @nodelist = $parent->getElementsByTagName( $tag );

  foreach $a ( @nodelist ) {
    if ( $a->getAttributes( )->getNamedItem( $name_attr )->getValue( ) eq $name ) {
      return $a;
    }
  }
}

sub _get_named_node_attr {
  # match name_attr attribute if supplied.
  my ( $parent, $tag, $name, $attr, $name_attr ) = @_;
  # otherwise use the name attribute.
  $name_attr ||= "name";

  _get_named_node( $parent, $tag, $name, $name_attr )->getAttributes( )->getNamedItem( $attr )->getValue( );
}

sub _set_named_node_attr {
  # match name_attr attribute if supplied.
  my ( $parent, $tag, $name, $attr, $valu, $name_attr ) = @_;
  # otherwise use the name attribute.
  $name_attr ||= "name";

  _get_named_node( $parent, $tag, $name, $name_attr )->getAttributes( )->getNamedItem( $attr )->setValue( $valu );
}

sub _remove_named_node {
  # match name_attr attribute if supplied.
  my ( $parent, $tag, $name, $name_attr ) = @_;
  # otherwise use the name attribute.
  $name_attr ||= "name";

  if ( _get_named_node( $parent, $tag, $name, $name_attr ) ) {
    $parent->removeChild( _get_named_node( $parent, $tag, $name, $name_attr ) );
  }
}

sub dispose {
  $_d->dispose( );
}

1;
