# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::ResourceMonitor;
# File: VMware/Management/ResourceMonitor.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials
                                 append_error
                                 append_agent
                                 append_newMui
                                 connect_ctrl_obj );

use XML::DOM ();
use XML::Sablotron ();
use CGI ();

my $q;
my $edit_ok = 1;
my @errors;
my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();
  $q = new CGI(shift());
  my %credentials = hash_credentials();
  my ($server, $server_ok, $err, $errstr);
  my $parser = new XML::DOM::Parser;
  my $version = VMware::Control::version();
  my $processor = new XML::Sablotron();
  my $want_version = "1.00";
  my $xsl = $ENV{ "style_DEFAULT" };
  my $xml;

  undef @errors;

  $server = &VMware::Control::Server::new(undef, 0, $credentials{"l"}, $credentials{"m"});
  ($server_ok, $err, $errstr) = connect_ctrl_obj($server);

  if ($server_ok) {
    my $ret;

    trace("edit=".$q->param("edit"), "debug");

    if ($ENV{"PATH_INFO"} eq "/editor" && !$q->param("edit")) {
      my $cfg = $q->param( "c" );
      $cfg ||= $q->param( "vm" );

      trace("World -> $cfg", "debug");

      $ret = $server->exec("<exec><op>ESXResourceInfo_GetWorld</op><in><world>$cfg</world></in></exec>");
    } elsif ($ENV{"PATH_INFO"} eq "/editor" && $q->param("edit")) {
      my $cfg = $q->param( "c" );
      $cfg ||= $q->param( "vm" );
      $cfg ||= $q->param( "world" );

      my $numCPU = $q->param("numCPU");
      my $cpu_shares = $q->param("cpu_shares");
      my $cpu_min = $q->param("cpu_min");
      my $cpu_max = $q->param("cpu_max");
      my $cpu_affinity = getCPUAffinity($q);
      my $memory_shares = $q->param("memory_shares");
      my $memory_min = $q->param("memory_min");
      my $memory_max = $q->param("memory_max");
      my $memory_affinity = getMemAffinity($q);
      my $has_filter = $q->param("has_filter");
      my $enable_filter = $q->param("enable_filter");
      my $avg_bandwidth = $q->param("avg_bandwidth");
      my $peak_bandwidth = $q->param("peak_bandwidth");
      my $burst_size = $q->param("burst_size");

      trace("Edit World -> $cfg", "debug");
      trace("CPU Shares -> $cpu_shares", "debug");
      trace("CPU Min -> $cpu_min", "debug");
      trace("CPU Max -> $cpu_max", "debug");
      trace("CPUs -> $numCPU", "debug");
      trace("CPU Affinity -> $cpu_affinity", "debug");
      trace("Memory Shares -> $memory_shares", "debug");
      trace("Memory Min -> $memory_min", "debug");
      trace("Memory Max -> $memory_max", "debug");
      trace("Memory Affinity -> $memory_affinity", "debug");
      trace("has_filter -> $has_filter", "debug");
      trace("enable_filter -> $enable_filter", "debug");
      trace("avg_bandwidth -> $avg_bandwidth", "debug");
      trace("peak_bandwidth -> $peak_bandwidth", "debug");
      trace("burst_size -> $burst_size", "debug");

      if (defined($cpu_shares) && ($cpu_shares ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetCPUShares</op><in><world>$cfg</world><shares>$cpu_shares</shares></in></exec>"));
      }

      if (defined($cpu_min) && ($cpu_min ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetCPUMin</op><in><world>$cfg</world><min>$cpu_min</min></in></exec>"));
      }

      if (defined($cpu_max) && ($cpu_max ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetCPUMax</op><in><world>$cfg</world><max>$cpu_max</max></in></exec>"));
      }

      if (defined($cpu_affinity) && ($cpu_affinity ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetCPUAffinity</op><in><world>$cfg</world><affinity>$cpu_affinity</affinity></in></exec>"));
      }

      if (defined($memory_shares) && ($memory_shares ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetMemShares</op><in><world>$cfg</world><shares>$memory_shares</shares></in></exec>"));
      }

      if (defined($memory_min) && ($memory_min ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetMemMin</op><in><world>$cfg</world><min>$memory_min</min></in></exec>"));
      }

      if (defined($memory_max) && ($memory_max ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetMemMax</op><in><world>$cfg</world><max>$memory_max</max></in></exec>"));
      }

      if (defined($memory_affinity) && ($memory_affinity ne "")) {
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetMemAffinity</op><in><world>$cfg</world><affinity>$memory_affinity</affinity></in></exec>"));
      }

      # Disk resource management
      foreach my $key ( grep( /^disk_shares_/, $q->param) ) {
          $key =~ /^disk_shares_(.*)$/;
          my $target = $1;
          my $shares = $q->param($key);
          trace("Disk Shares ($target) -> $shares", "debug");
          if (defined($shares) && ($shares ne "")) {
              trackEditErrors($server->exec("<exec><op>ESXResourceEdit_SetDiskShares</op>" .
                                            "<in><world>$cfg</world><target>$target</target>" .
                                            "<shares>$shares</shares></in></exec>"));
          }
      }

      # Four states:
      #  1) have a filter, we wish to disable it
      #  2) no current filter, we wish to enable one
      #  3) have a filter, we change it
      #  4) no current filter, no wish to enable it.  Do nothing.
      if ($has_filter && !$enable_filter) {
	# 1) have a filter, we wish to disable it
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_DetachNetFilter</op><in><world>$cfg</world></in></exec>"));
      } elsif( !$has_filter && $enable_filter ) {
        # 2) no current filter, we wish to enable one
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_AttachNetFilter</op><in><world>$cfg</world><nfclass>nfshaper</nfclass><args>".$avg_bandwidth."M ".$peak_bandwidth."M ".$burst_size."k</args></in></exec>"));
      } elsif( $has_filter && $enable_filter ) {
	# 3) have a filter, we change it
        trackEditErrors($server->exec("<exec><op>ESXResourceEdit_ChangeNetFilter</op><in><world>$cfg</world><instance>nfshaper.x.y</instance><args>".$avg_bandwidth."M ".$peak_bandwidth."M ".$burst_size."k</args></in></exec>"));
      }

      # After configuring the appropriate resources, grab the world XML.
      $ret = $server->exec("<exec><op>ESXResourceInfo_GetWorld</op><in><world>$cfg</world></in></exec>");
    } else {
      $ret = $server->exec("<exec><op>ESXResourceInfo_GetAll</op><in></in></exec>");
    }

    if (defined($ret)) {
      my $mui = $q->param( "mui" );
      $ret = VMware::DOMAccess->newXML($ret);

      # XXX It's possible that serverd should be including this in
      # exec.out.server rather than inserting it here.
      append_agent($ret->getDocument(), $ret->get("exec.out.server"));
      if ($mui) {
         append_newMui($ret->getDocument(), $ret->get("exec.out.server"), $mui);
      }

      my $sub = $ret->getSubTree("exec.out.server");

      if (!$edit_ok) {
        foreach my $error (@errors) {
          if(defined($error)) {
            $sub->addSubTree("server", $error);
            $error->dispose();
          }
        }
      } elsif ($q->param("edit")) {
        $sub->addSubTree("server", VMware::DOMAccess->newXML("<success/>"));
      }

      $xml = $sub->toEncodedString('');
      $sub->dispose();
      $ret->dispose();
    }
  } else {
    # rbosch: This is an ESX-only page, make sure we get banner right
    $xml = qq{<server><product id="ESX"/></server>};
    # [ ! ] XXX Trace an error about the inability to connect to vmauthd
    trace("$err: $errstr", "error");
  }

  if ($server_ok) {
    $server->disconnect( );
    trace( "server->disconnect() called ($server).", "notice");
  }

  if ($ENV{ "vmware_DEBUG_LEVEL" } > 2) {
    my $xmlfile;
    if (defined($ENV{"PATH_INFO"}) && $ENV{"PATH_INFO"} ne "/") {
      $_ = $ENV{"PATH_INFO"};
      s/\/(.*?)\/?/$1/g;
      $xmlfile = &VMware::Management::Util::tmpdir() . "resources$_.xml";
    } else {
      $xmlfile = &VMware::Management::Util::tmpdir() . "resources.xml";
    }
    string_to_file($xml, $xmlfile, 1);
  }

  $processor->SetBase("file://" . $ENV{ "vmware_BASEHREF" });
  $p->start("XML::Sablotron::RunProcessor");
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template",
                            file_to_string($xsl),
                            "data",
                            $xml ] );
  $p->end();

  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache, must-revalidate",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

sub getCPUAffinity {
  my $q = shift;

  # Undefined numCPU means we can't set CPU affinity for this world
  my $numCPU = $q->param("numCPU");
  if (!defined($numCPU)) {
    return undef;
  }

  my @cpu_affinity = ();
  for my $id (0 .. ($numCPU - 1)) {
    my $affinity = $q->param("cpu_affinity-${id}");
    if (defined($affinity) && ($affinity ne "") && ($affinity < $numCPU)) {
      push @cpu_affinity, $affinity;
    }
  }

  # If no affinity is set or affinity is set for all processors, use
  # the "all" keyword instead of a list.
  my $count = scalar(@cpu_affinity);
  if (($count == 0) || ($count == $numCPU)) {
    return "all";
  } else {
    return join(',', @cpu_affinity);
  }
}

sub getMemAffinity {
  my $q = shift;
  
  # Undefined numNode means we can't set memory affinity for this world
  my $numNode = $q->param("numNode");
  if (!defined($numNode)) {
    return undef;
  }

  my @mem_affinity = ();
  for my $id (0 .. ($numNode - 1)) {
    my $affinity = $q->param("mem_affinity-${id}"); 
    if (defined($affinity) && ($affinity ne "") && ($affinity < $numNode)) {
      push @mem_affinity, $affinity;
    }
  }

  # If no affinity is set or affinity is set for all nodes, use
  # the "all" keyword instead of a list.
  my $count = scalar(@mem_affinity);
  if (($count == 0) || ($count == $numNode)) {
    return "all";
  } else {
    return join(',', @mem_affinity);
  }
}

sub trackEditErrors {
  my $xml = shift(); # Raw XML from an exec call
  my $ret = VMware::DOMAccess->newXML($xml);
  if (!$ret->getValue("exec.returncode")) {
    $edit_ok = 0;
    $errors[scalar(@errors)] = $ret->getSubTree("exec.messages");
    trace($xml, "debug");
  }
}

1;
