#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# $Id$
#############################################################
#
# Index.pm
# 
# A handler that routes the initial URL to the correct location
package VMware::Management::SS::Index;

use strict;
use VMware::ExtHelpers qw(&clearWarnings);
use VMware::DOMAccess;
use VMware::HConfig::Constant qw($URL_INDEX $URL_OVERVIEW $URL_LOGOUT
				 );
use VMware::Management::Util qw(hash_credentials );
use VMware::HConfig::SSHostRemote;

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "VMware ESX Server Configuration";


sub toErrorHTML($$) {
   my ($eSummary, $eDesc) = @_;
   return toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLError($eSummary, $eDesc);
}

#####################################################################
# End of Serverd Remote Request Support Functions                   #
#####################################################################

sub handler {
   my $url = $URL_INDEX;
   my $fail = undef;

   &clearWarnings();
   
   my $cgi = new CGI( shift( ) );

   my %credentials = hash_credentials( );
   my $user = $credentials{ "l" };
   my $password = $credentials{ "m" };

   my @path = ();

   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);
   if (!defined($host)) {
      $fail = toHTMLHostFailure($htmlTitle);
      goto fail;
   }

   if ($host->isVMKernel()) {
      # Redirect to overview if the system is setup properly. */
      $url = $URL_OVERVIEW;
   } else {
      if( $user ne 'root' ) {
         $fail = toErrorHTML("System not configured",
                             "Your system cannot run virtual machines until the vmkernel is\n" .
                             "loaded.  If you have not finished configuring your machine, please\n" .
                             "<a href=\"$URL_LOGOUT\">logout</a> and log in as the root user.\n");
         goto fail;
      }
   }

   return ( "&nbsp",
            "Pragma"        => "no-cache",
            "Cache-Control" => "no-cache",
            "Location"  => $url);
   

 fail:
   my $s = toHTMLHeader($htmlTitle, $user);
   $s .= $fail;
   $s .= toHTMLFooter();
   return ($s, %MimeHeaders);
}

1;
