#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# SNMPConfig.pm
#
# A script that allows users to configure SNMP access
package VMware::Management::SS::SNMPConfig;

use strict;

use VMware::HConfig::SSHostRemote;
use VMware::ExtHelpers qw( &Warning &clearWarnings );
use VMware::Management::Util qw( &hash_credentials
                                 trace );

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "SNMP Configuration";

my $RCom = "request";
my $RCom_Save = "save";
my $mui; # will be set as 'new' if the new MUI is being used
my $url_self;

use CGI;
use Carp qw(confess);
$SIG{__DIE__} = \&confess;

sub toSNMPConfigHTML($$) {
   my ($cgi, $host) = @_;
   my $wizard = $cgi->param("wizard");

   my $config = $host->getSNMPConfig();
   my $enable = $host->getSNMPEnable();

   my $enabled = (defined($enable) && $enable) ? "checked" : "";
   my $disabled = (defined($enable) && !$enable) ? "checked" : "";

   my $txtAreaCols = ($mui eq "new") ? 70: 80;
   my $htmlSubBtn  = ($mui eq "new") ? "": '<input type="submit" value="Save Changes">';
   my $htmlCtx     = ($mui eq "new") ? "OK": "Save Changes";
   my $s = <<END_HTML;
<h3>SNMP Configuration</h3>

<p class="txt">
VMware ESX Server ships with an SNMP agent that allows you to monitor the health of
the host machine and of virtual machines running on the host.  The ESX Server SNMP
agent can be used with any management software that can load and compile a management
information base (MIB) in SMIv1 format and can understand SNMPv1 trap messages.
</p>

<p class="txt">
Make any desired changes to the SNMP configuration below,
and click "$htmlCtx" to save the new configuration.
</p>

<form method="POST" action="$url_self" name="snmp">

<h4>SNMP Enable</h4>

<table>
  <tr><td class="txt"><input type="radio" name="enable" value="1" $enabled>
    Enable SNMP agent and SNMP trap generation
  </td></tr>
  <tr><td class="txt"><input type="radio" name="enable" value="0" $disabled>
    Disable SNMP agent and SNMP trap generation
  </td></tr>
</table>

<h4>SNMP Configuration Options</h4>

<p class="txt">
For more information on the SNMP configuration options, please consult the
VMware ESX Server Administration Guide.
</p>

<center><tt>
  <textarea name="data" rows="25" cols="$txtAreaCols">$config</textarea>
</tt></center>

<br>

<input type="hidden" name="mui" value="$mui">
<input type="hidden" name="$RCom" value="$RCom_Save">
<input type="hidden" name="wizard" value="$wizard">
$htmlSubBtn

</form>
END_HTML

   return($s);
}

sub updateSNMPConfig {
   my ($cgi, $host) = @_;
   my $config = $cgi->param("data");
   my $enable = $cgi->param("enable");

   my $ret = $host->writeSNMPConfig($config, $enable);
   if (!$ret) {
      Warning("Error writing SNMP configuration: file is unchanged.");
   }
}


sub newMuiHeader {
   return <<END_HTML_HEADER;
<html>

<head>
  <title>SNMP Configuration!</title>
  <link rel="stylesheet"
        href="/vmware/themes/default/style.css"
        type="text/css">


  <script type="text/javascript">
    <!--
      function aggressiveUpdate() {
        window.setTimeout('window.location.reload( )', 1000 * 60);
      }
      function openHelp(ref,w,h) {
         window.open(ref,
                     "vmware_mui_help",
                     "location,menubar,resizable,scrollbars,status,toolbar," +
                     "width=" + w + ",height=" + h);
         aggressiveUpdate();
      }
      function next() {
        document.snmp.action="$url_self";
        document.snmp.submit();
      }
    // -->
  </script>
</head>

<body bgcolor="#ffffff" topmargin="6" leftmargin="6" marginwidth="6" marginheight="6"
      link="#3366cc" vlink="#663300">
END_HTML_HEADER
}


sub handler {
   &clearWarnings();

   my $cgi = new CGI( shift( ) );
   my %credentials = hash_credentials( );

   my $user = $credentials{ "l" };
   my $password = $credentials{ "m" };

   my $s;
   $url_self = $cgi->url(-relative=>1);

   # variable used to hide content not necessary within the new MUI frame
   if ($mui = $cgi->param("mui")) {
      trace("New MUI set.", "debug");
   }

   # get HTML Header Content
   if (!($mui eq "new")) {
      $s = toHTMLHeader($htmlTitle, $user) . toHTMLPath(\@htmlPath, $htmlTitle);
   } else {
      $s = newMuiHeader();
   }

   if ($user ne 'root') {
      $s .= toHTMLNotRoot($url_self);
      goto footer;
   }

   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);
   if (!defined($host)) {
      $s .= toHTMLHostFailure($htmlTitle);
      goto footer;
   }

   my $req = $cgi->param($RCom);
   if (defined($req) && ($req eq $RCom_Save)) {
       updateSNMPConfig($cgi, $host);
   }

   $s .=
       toHTMLSystemInfo($host) .
       toHTMLWarnings() .
       toSNMPConfigHTML($cgi, $host);

   # if in new mui & the form submitted w/o errors, close the child window
   my $submitErrs = toHTMLWarnings();
   if (defined($req) && ($req eq $RCom_Save) &&
   ($submitErrs !~ "Reported Warnings") && ($mui eq "new")) {
      $s .= "<script>parent.close();</script>";
   }

footer:
   # get HTML Footer Content
   if (!($mui eq "new")) {
      $s .= toHTMLFooter();
   }

   return ($s, %MimeHeaders);
}

1;
