package VMware::Management::SS::ScriptedInstall::Lib;

use VMware::HConfig::Constant qw( %EsxScriptedInstall );

require Exporter;
@ISA = qw(Exporter);

## Sorta odd path because of the caller, which is ScriptedInstall.pm
BEGIN {
   push(@INC, '/usr/lib/vmware-mui/lib/perl5/site_perl/5.005/VMware/Management/SS/ScriptedInstall');
}

use RedHat::Kickstart;

@EXPORT = qw( buildKickstartFile 
              buildKickstartDiskImage
              doScriptedInstallSetup );

@EXPORT_OK = qw( readFile
                 writeFile
                 checkValidIPAddress 
                 getSystemSettings );


sub readFile {
   my $fileName = shift;
 
   my $rv = open (FILE, "<$fileName"); 

   if (!$rv) {
      warn "Couldn't open file '$fileName' for reading: $!";
      return;
   }
 
   my @file = <FILE>;
   close FILE;
 
   return join '', @file;
}

sub writeFile {
   my $fileName = shift;
   my $data = shift;
   my $binaryMode = shift;
 
   my $rv = open (FILE, ">$fileName");
   
   if (!$rv) {
      warn "Couldn't open file '$fileName' for writing: $!";
      return;
   }

   binmode(FILE) if ($binaryMode);
   print FILE $data;
   close FILE;

   return 1;
}

sub checkValidIPAddress {
  my $ip = shift;

  return if ($ip !~ /^ \d{1,3} \. \d{1,3} \. \d{1,3} \. \d{1,3} $/x);

  foreach my $octet (split(/\./, $ip)) {
    return if (int($octet) < 0 or int($octet) > 255);
  }

  return 1;
}

sub buildKickstartFile {
   my $cgiObj = shift;
   my $ksFile = new RedHat::Kickstart;

   #
   # Set our default system settings
   #

   ## We don't modify this right now
   if($cgiObj->param('keyboard')) {
      $ksFile->keyboard($cgiObj->param('keyboard'));
   } else {
      $ksFile->keyboard(getSystemSettings(
       $EsxScriptedInstall{'SETTINGS'}->{'keyboard'}));
   }

   if($cgiObj->param('tz')) {
      $ksFile->timezone($cgiObj->param('tz'));
   } else {
      $ksFile->timezone(getSystemSettings(
       $EsxScriptedInstall{'SETTINGS'}->{'tz'}));
   }

   my $lang = getSystemSettings($EsxScriptedInstall{'SETTINGS'}->{'i18n'});
   $lang->{'default'} = $lang->{'lang'};
   $lang->{'langsupport'} = $lang->{'lang'};

   $ksFile->lang($lang);
   $ksFile->langsupport($lang);

   # $ks_obj->mouse(getSystemSettings(
   #  $EsxScriptedInstall{'SETTINGS'}->{'mouse'} ));
   $ksFile->mouse('none');

   #
   # Set everything else
   #
 
   if($cgiObj->param('install_method') eq 'cdrom' ) {
      $ksFile->inst_type( 'cdrom' );
   } elsif($cgiObj->param('install_method') eq 'remote') {
      my $url = $cgiObj->param('remote_server');
      $ksFile->inst_type( {
          inst_type       => 'url',
          url             => $url
      });
  
   } elsif ($cgiObj->param('install_method') eq 'lan' ) {
         my $hostname = `hostname -f` or die "Hostname must be set: couldn't resolve it: $!\n";
         $ksFile->inst_type( {
         inst_type       => 'url',
         url             => "http://$hostname:$EsxScriptedInstall{'PORT'}/" });
   }

   $ksFile->install($cgiObj->param('install_type'));

   $ksFile->rootpw($cgiObj->param('passwd1')) if $cgiObj->param('passwd1');

   $ksFile->lilo( { 'location' =>
                  ( $cgiObj->param('mbr') eq 'custom' ?
                    $cgiObj->param('mbr_partition') : 'mbr' )
                  } );

   if($cgiObj->param('network') eq 'static') {
      $ksFile->network( {
       bootproto       => 'static',
       ip              => $cgiObj->param('ipaddress'),
       netmask         => $cgiObj->param('netmask'),
       gateway         => $cgiObj->param('defaultgw'),
       nameserver      => $cgiObj->param('nameserver1'), });

      $ksFile->network({ hostname => $cgiObj->param('hostname') } )
       if $cgiObj->param('hostname');
   } else {
      $ksFile->network('dhcp');
   }

  $ksFile->package('@ ESX Server');

  $ksFile->reboot(1) if ($cgiObj->param('reboot') eq 'Yes');

  $ksFile->driverdisk( { partition => '/dev/fd0' } )
    if ($cgiObj->param('driverdisk'));

  for (my $ndx = 0; $ndx < $EsxScriptedInstall{'MAX_PARTITIONS'}; $ndx++) {
    next unless ($cgiObj->param("size$ndx") =~ /^[1-9]\d*$/);
  
     $ksFile->partition( {
          dir            => $cgiObj->param("mount$ndx" ),
          size           => $cgiObj->param("size$ndx"),
          ondisk         => $cgiObj->param("drive$ndx"),
          grow           => ( $cgiObj->param("grow$ndx") ? 1 : 0 )
     });
   }

   $ksFile->clearpart( 'all' );

   $ksFile->text( 1 );
   $ksFile->auth( { 'useshadow' => 1, 'enablemd5' => 1 } );

   my $post_install = readFile(
    "$EsxScriptedInstall{'WEBCONFIG_ROOT'}/$EsxScriptedInstall{'KS_FILE'}");

   $post_install =~ s/ ^ .* \%post \s* (?:--nochroot|)//xs;

   $ksFile->post( { nochroot => 1, post => $post_install } );

   return $ksFile;
}

sub getSystemSettings {
  my $inputHash = shift;

  my $file = $inputHash->{'file'};
  my $translate = $inputHash->{'translate'};

  my $rv = open(CFGFILE, "<$inputHash->{'file'}");
  if (!$rv) {
     warn "Couldn't open $inputHash->{'file'}: $!";
     return; 
  }

  my @cfgFile = <CFGFILE>;
  close(CFGFILE);

  my $outCfg = {};

  foreach my $line (@cfgFile) {
    chomp $line;

    my($key, $value)  = split('=', $line);

    $key = lc($key);
    $value =~ s/\"//g;

    # map any system settings with different names to
    # their appropriate kickstart key name

    $key = $translate->{$key} if exists($translate->{$key});

    $outCfg->{$key} = $value;
  }

  return $outCfg;
}

sub buildKickstartDiskImage {
  my $cgiObj = shift;
  my $rv;

  ## Error checking

  my $ks_obj = buildKickstartFile($cgiObj);

  my $ks_file_name = `mktemp /tmp/autoinstall-ks.XXXXXX`;
  my $tmp_file_name = `mktemp /tmp/autoinstall-img.XXXXXX`;

  return $EsxScriptedInstall{'ERRORS'}{'ETMPNAME'} 
   if (not $ks_file_name or not $tmp_file_name); 

  chomp($ks_file_name);
  chomp($tmp_file_name);

  my $ks_data = ${ $ks_obj->kickstart($EsxScriptedInstall{'KS_VERSION'}) };

  my $binaryMode = 1; ## Turns on binary mode on the FH inside the func 
  writeFile($ks_file_name, $ks_data, $binaryMode);

  if( $cgiObj->param('install_method') eq 'cdrom' ) {
     $rv = `/bin/cp $EsxScriptedInstall{'BOOTIMG_LOCATION'} $tmp_file_name`;
  } else {
     $rv = `/bin/cp $EsxScriptedInstall{'BOOTIMG_NET_LOCATION'} $tmp_file_name`;
  }

  $rv = `/usr/bin/sudo $EsxScriptedInstall{'WEBCONFIG_ROOT'}/modifyboot --ks=$ks_file_name --file=$tmp_file_name --syslinux=syslinux.cfg`;

  my $ks_image = readFile( $tmp_file_name );

  unlink($tmp_file_name);
  unlink($ks_file_name);

  return $ks_image;
}

sub doScriptedInstallSetup() {
   die "Not implemented.";
}

1;
