#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# UptimeReport.pm
# 
# A script that displays an uptime report for the server.
#
package VMware::Management::SS::UptimeReport;

use strict;
use VMware::HConfig::SSHostRemote;
use VMware::ExtHelpers qw(&clearWarnings);
use Carp qw(confess);
$SIG{__DIE__} = \&confess;
use CGI;
use VMware::Management::Util qw( hash_credentials );
use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Availability Report";

my $hostname;
my $port;
my $user;
my $password;


#
#  The actual report is generated via a cron job, just ask
# serverd to give us the html and display it.
#
sub handleRequest($$) {
   my ($cgi, $host) = @_;
   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("File_Read");

   $in->setValue('.in.file', '/var/log/vmksummary.html');

   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname, $port, $user, $password, $in);
   my $s = '';

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (!$rc) {
      &VMware::HConfig::SSHostRemote::ServerdWarning($out);
      $s .= toHTMLWarnings();
   } else {
      $s = $out->getValue('.out.data');
   }
   return $s;
}

sub handler {
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'}); 

    &clearWarnings();

    my $cgi = new CGI( shift( ) );
    my %credentials = hash_credentials( );

    $hostname = undef;
    $port = 0;
    $user = $credentials{ "l" };
    $password = $credentials{ "m" };

    my $s = toHTMLHeader($htmlTitle, $user);

    if ($user ne 'root') {
       my $url_self = $cgi->url(-relative=>1);
       $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLNotRoot($url_self);
       goto footer;
    }
    
    my $host = VMware::HConfig::SSHostRemote->new($hostname, 0, $user, $password);

    if (!defined($host)) {
       $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLHostFailure($htmlTitle);
       goto footer;
    }

    $s .= handleRequest($cgi, $host);

footer:
    $s .= toHTMLFooter();
    return ($s, %MimeHeaders);
}

1;
