#!/usr/bin/perl
# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::VMFSBrowse;
# File: VMware/Management/VMFSBrowse.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::DOMAccess;

use CGI;

use XML::DOM;
use XML::Sablotron;

my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
    my $stub = $p->profile();
    my $doc;
    my $root;
    my $ret;
    my $xml;
    my $cgi = new CGI(shift());
    my $xsl = $ENV{ "style_DEFAULT" };
    my $parser = new XML::DOM::Parser;
    my %credentials = hash_credentials( );
    my $server = &VMware::Control::Server::new( undef, 0,
                                              $credentials{ "l" },
                                              $credentials{ "m" } );
    $server->connect();

    my $doc = VMware::DOMAccess->new("server");
    my $perm = $cgi->param("permission");

    my $query = VMware::DOMAccess->new("exec");
    $query->setValue("exec.op", "ESXHostInfo_GetVMFS");
    if (defined($perm) && ($perm ne "")) {
	$query->setValue("exec.in.permission", $perm);
    }
    $xml = $server->exec($query->toEncodedString(""));
    $query->dispose();
    
    $ret = VMware::DOMAccess->newXML($xml);

    my @names = $ret->listElementNames("exec.out");
    my $n;
    foreach $n (@names) {
	my $s = $ret->getSubTree("exec.out." . $n);
	$s = VMware::DOMAccess->newXML($s->getValue(""));
	$doc->addSubTree("server", $s);
	$s->dispose();
    }
    $ret->dispose();

    foreach my $key ("extension", "defaultSize", "defaultName") {
       my $val = $cgi->param($key);
       if (defined($val) && ($val ne "")) {
          $doc->setValue(".$key", $val);
       }
    }

    my $processor = new XML::Sablotron();

    $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
    $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                              undef,
                              [ "template", file_to_string( $xsl ), "data", $doc->prettyPrint() ] );
    $p->end();
    $doc->dispose();

    return ( $processor->GetResultArg("result"),
           "Content-Type"  => "text/html; charset=iso-8859-1",
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache" );
}

1;
