#!/usr/bin/perl

package VMware::Management::SS::VMKernelUpdate;
# File: VMware/Management/SS/VMKernelUpdate.pm

use strict;
use VMware::Config;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Management::Util;
use VMware::DOMAccess;
use CGI;
use URI ();

use XML::DOM;
use XML::Sablotron;

my $mui; # will be set as 'new' if the new MUI is being used


sub handler {
  my $cgi = new CGI(shift());

  my $req = $cgi->param('request');

  # variable used to hide content not necessary within the new MUI frame
  $mui = $cgi->param('mui');
  if ($mui eq "new") {
    trace("New MUI set.", "debug");
  }

  if (defined($req) && $req eq 'update') {
    return handleUpdate($cgi);
  } else {
    return handleUpdateInfo($cgi);
  }
}


sub handleUpdateInfo($) {
  my $cgi = shift;
  my $param = $cgi->param('p');
  my $value = $cgi->param('v');
  my $default = $cgi->param('d');

  my $processor = new XML::Sablotron();
  my $xsl = $ENV{ "style_DEFAULT" };

  my $xml  = "<server><param>$param</param><value>$value</value>";
     $xml .= "<default>$default</default><mui>$mui</mui></server>";

  $processor->SetBase("file://" . $ENV{ "vmware_BASEHREF" });
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template",
                              file_to_string($xsl),
                              "data",
                              $xml ] );

  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache, must-revalidate",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

sub handleUpdate($) {
  my $cgi = shift;
  my $param = $cgi->param('p');
  my $value = $cgi->param('v');

  # Value field is blank: do nothing
  if ( $value eq "" ) {
      return toUpdateSuccessHTML();
  }

  my $ret;
  my %credentials = hash_credentials();
  my $server = &VMware::Control::Server::new(undef, 0,
                                             $credentials{"l"},
                                             $credentials{"m"});

  if ( defined( $server ) && $server->connect( ) ) {
    trace( "Connected to server object ($server).", "notice" );

    my $doc = VMware::DOMAccess->new('exec');
    $doc->setValue('.op', 'ESXVMKernelEdit_SetParam');
    $doc->setValue('.in.param.name', $param);
    $doc->setValue('.in.param.value', $value);

    my $request = $doc->toEncodedString('');
    $ret = $server->exec($request);
    $server->disconnect();
    $doc->dispose();
  }

  my @errors;
  if (defined($ret)) {
    $ret = VMware::DOMAccess->newXML($ret);
    if ($ret->getValue("exec.returncode") == 1) {
      $ret->dispose();
      return toUpdateSuccessHTML();
    } else {
      foreach my $i ($ret->listElementNames("exec.messages")) {
        push @errors, $ret->getValue("exec.messages.$i");
      }
      $ret->dispose();
    }
  } else {
    push @errors, "Could not connect to server";
  }
  return toUpdateFailureHTML(@errors);
}

sub toUpdateSuccessHTML() {
  my $muiParam = ($mui eq "new") ? "?mui=new": "";
  my $s = <<END_HTML;
<html>
  <head>
    <title>Update Success Dialog</title>
    <SCRIPT language="JavaScript">
       function UpdateSuccess() \{
          opener.location.href = "/vmkernel-config$muiParam";
          self.close();
       \}
    </SCRIPT>
  </head>
  <body bgcolor="#FFFFE6" onload="UpdateSuccess()">
    <p class='txt'>
      VMKernel parameter update succeeded.  This window
      should be closed.  If it is not closed, follow this
      <a href="Javascript:UpdateSuccess()">link</a>.
    </p>
  </body>
  </html>
END_HTML

  return ( $s,
           "Content-Type"  => "text/html; charset=iso-8859-1",
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache" );
}

sub toUpdateFailureHTML() {
  my $errors = "<ul>\n" . join('\n', map { qq(<li class='txt'>$_) } @_) . "</ul>\n";
  my $muiParam = ($mui eq "new") ? "?mui=new": "";
  my $s = <<END_HTML;
<html>
  <head>
    <title>VMkernel Update Failure</title>
    <link rel="stylesheet" href="/vmware/themes/default/style.css" type="text/css">
    <SCRIPT language="JavaScript">
       function CloseWindow() \{
          opener.location.href = "/vmkernel-config$muiParam";
          self.close();
       \}
    </SCRIPT>
  </head>
  <body bgcolor="#FFFFE6">
    <p class='txt'>VMKernel parameter update failed with the following errors:</p>
      $errors
    <p class='txt'>Click <a href="Javascript:CloseWindow()">here</a> to continue.</p>
  </body>
  </html>
END_HTML

  return ( $s,
           "Content-Type"  => "text/html; charset=iso-8859-1",
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache" );
}

1;
