# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxNetwork;
# File: VMware/Management/SxNetwork.pm

use strict;

use VMware::Management::Util ();
use VMware::Management::VmdbUtil ();

use CGI();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();

   if ($q->param("op") =~ /^(save)$/i) {
      return save($ctx);
   }

   return data($ctx);
}

# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift() || "";
   my $path;

   #
   # Network configuration: /host2/#/config/network/
   #

   $ctx->SetCurrentPath("/host2/#_mui/config/network/");

   # Virtual Network names

   my %netName;
   $path = "#/";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device/");
      my $name = $ctx->GetString("${path}network/");
      $netName{$dev} = $name;
   }

   #
   # Network state: /host2/#/vmnetState/
   #

   $ctx->SetCurrentPath("/host2/#_mui/vmnetState/");

   # Physical adapter information

   # JavaScript constructor for NIC object
   $data .= "function Nic(name, type, id, vnet, mac) {" .
       "this.name = name;" .         # Human-readable device name
       "this.vnet = vnet;" .         # VNet of device 
       "this.id = id;" .             # ID of device 
       "this.type = type;" .         # Type of device (physical/virtual)
       "this.mac = mac;" .           # MAC address of device
       "}$/";

   $data .= "var nics = new Array();$/";
   $path = "val/nic/#/";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $type = jsStringLiteral($ctx->GetString("${path}type/"));
      if ($type eq "physical") {
         my $name = jsStringLiteral($ctx->GetString("${path}printName/"));
         my $id = $ctx->GetInt("${path}id/");
         my $vnet = $ctx->GetInt("${path}vnet/");
         my $mac = jsStringLiteral($ctx->GetString("${path}macAddress/"));
         $data .= "nics.push(new Nic($name, $type, $id, $vnet, $mac));$/";
      }
   }

   # Virtual Network configuration

   # JavaScript constructor for VMnet object
   $data .= "function Net(dev, name, type, vnet, present, ip) {" .
       "this.dev = dev;" .           # Device name
       "this.name = name;" .         # Human-readable ('friendly') device name
       "this.vnet = vnet;" .         # VNet of device
       "this.type = type;" .         # Type of device (bridged/nat/hostonly/disabled)
       "this.present = present;" .   # Is device present?
       "this.ip = ip;" .             # IP of device (if applicable)
       "}$/";

   $data .= "var nets = new Array();$/";
   $path = "val/vmnet/#/";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}name/");
      my $name = $netName{$dev} or "";
      $dev = jsStringLiteral($dev);
      $name = jsStringLiteral($name);

      my $type;
      if ($ctx->GetBool("${path}bridged/")) {
         $type = "bridged";
      } elsif ($ctx->GetBool("${path}nat/present/")) {
         $type = "nat";
      } elsif ($ctx->GetBool("${path}adapter/present/")) {
         $type = "hostonly";
      } else {
         $type = "disabled";
      }
      $type = jsStringLiteral($type);

      my $present = jsBoolLiteral($ctx->GetBool("${path}present/"));
      my $vnet = $ctx->GetInt("${path}vnet/");
      my $ip = jsStringLiteral($ctx->GetString("${path}adapter/ipAddress"));

      $data .= "nets.push(new Net($dev, $name, $type, $vnet, $present, $ip));$/";
   }

   VMware::Management::Util::trace("$data", "debug");

   return html($data);
}


# ---------------------------------------------------------------------------

sub save {
   my $ctx = shift();
   my ($ok, $trnCtx, $reqPath);
   my $data = "";

   VMware::Management::Util::trace("Removing existing netmap devices...", "debug");

   # Track devices that we've already tried to handle to avoid any possibility
   # of an infinite loop.
   my %handled;

   # Handle existing NIC names.
   my $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $old = $ctx->GetString("${path}network");
      my $net = $q->param("$dev:net");

      if (! $handled{$dev} && (! defined($net) || '' eq $net)) {
         VMware::Management::Util::trace("Netmap device exists: $dev >$old<", "debug");

         # Track devices that we've already tried to handle to avoid any
         # possibility of an infinite loop.
         $q->delete("$dev:net");
         $handled{$dev} = 1;

         # T1(removeDevice) BC {
         #    T1(in) BC {
         #       T2(network, T_STR);
         #       T2(device, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "removeDevice");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/removeDevice/");

         $trnCtx->SetString("${reqPath}in/network", $old);
         $trnCtx->SetString("${reqPath}in/device", $dev);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
                           qq{Failed to remove mapping: "$dev" ~ "$old".});
         } else {
            # Be sure to start at the beginning since the /host2/#_mui/config/network
            # tree will be repopulated after the transaction completes
            # successfully.
            $path = "/host2/#_mui/config/network/#";
            VMware::Management::Util::trace("Removed netmap device: $dev >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   VMware::Management::Util::trace("Modifying existing netmap devices...", "debug");

   $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $old = $ctx->GetString("${path}network");
      my $net = $q->param("$dev:net");

      if (! $handled{$dev} && (defined($net) && $net ne "")) {
         VMware::Management::Util::trace("Netmap device exists: $dev >$old<", "debug");

         # Track devices that we've already tried to handle to avoid any
         # possibility of an infinite loop.
         $q->delete("$dev:net");
         $handled{$dev} = 1;

         if ($net eq $old) {
            VMware::Management::Util::trace("Existing netmap device unchanged: $dev >$old< : >$net<", "debug");
            next;
         }

         # T1(setNetwork) BC {
         #    T1(in) BC {
         #       T2(oldNetwork, T_STR);
         #       T2(newNetwork, T_STR);
         #       T2(newDevice, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "setNetwork");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/setNetwork/");

         $trnCtx->SetString("${reqPath}in/oldNetwork", $old);
         $trnCtx->SetString("${reqPath}in/newNetwork", $net);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
               qq{Failed to change mapping: "$dev" ~ "$net".});
         } else {
            # Be sure to start at the beginning since the /host2/#_mui/config/network
            # tree will be repopulated after the transaction complete
            # successfully.
            $path = "/host2/#_mui/config/network/#";
            VMware::Management::Util::trace("Modified netmap device: $dev >$old< : >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   VMware::Management::Util::trace("Adding new netmap devices...", "debug");

   # Handle new NIC names.
   for ($q->param()) {
      my $dev;

      if (/^(.*):net$/i) {
         $dev = $1;
      } else {
         next;
      }

      my $net = $q->param($_);

      if (defined($net) && $net ne "") {
         # T1(setNetwork) BC {
         #    T1(in) BC {
         #       T2(oldNetwork, T_STR);
         #       T2(newNetwork, T_STR);
         #       T2(newDevice, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "setNetwork");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/setNetwork/");

         $trnCtx->SetString("${reqPath}in/newNetwork", $net);
         $trnCtx->SetString("${reqPath}in/newDevice", $dev);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
                           qq{Failed to save mapping: "$dev" ~ "$net".});
         } else {
            VMware::Management::Util::trace("Added new netmap device: $dev >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $msg is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $msg = shift();
   $msg =~ s/\\/\\\\/g;
   $msg =~ s/("|')/\\$1/g;
   my $lf = $/;
   $msg =~ s/$lf/\\n/g;

   return qq{"$msg"};
}


# ---------------------------------------------------------------------------

sub jsBoolLiteral {
   return shift() ? "true" : "false";
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=new Array();var wrn=new Array();$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;

