# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxOperation;

use strict;

use CGI ();

use VMware::Management::Util ();
use VMware::Management::Util;
use VMware::VmdbPerl;
use VMware::Management::VmdbUtil ();
use VMware::Management::Session;

my $q;

sub handler {
   $q = new CGI(shift());

   my ($msg, $ok);
   my $errPath = "status/error/text/";

   my $vmId     = $q->param("vmid");

   my $data = qq{var data={};data.vmId="$vmId";};

   my $op = $q->param("op");
   my $opType   = $q->param("optype"); # 0 is soft, assume anything else is hard.

   my $ctx =  &VMware::Management::Session::get_temp_context();
   if (! defined($ctx)) {
      return html(error("Could not get temporary context.") . $data);
   }

   my $vmPath = "/vm/#$vmId/";

   # if valid context for a VM, then do the requested power operation
   if ($ctx->GetAbsPath($vmPath)) {
      $ctx->SetCurrentPath($vmPath);
      my ($trnCtx, $reqPath) = cmdHead($ctx, $op);
      # 0 is soft, assume anything else is hard.
      $trnCtx->SetString("${reqPath}in/softOption", $opType eq "0" ? "soft" : "hard");
      ($ok, $msg) = cmdTail($trnCtx, $reqPath);
   } else {
      ($ok, $msg) = (0, "Could not read path to virtual machine: #$vmId");
   }

   if (! $ok) {
      $msg ||= "Unknown error";

      # Handle multiline and one line messages.
      if ($msg =~ m{\Q$/\E}s) {
         $msg = "$/$/$msg";
      } else {
         $msg =~ s/(.*)\.$/$1/;
         $msg .= ". ";
      }

      if ($op eq "powerOn") {
         $msg = "The virtual machine did not power on: $msg";
      } elsif ($op eq "powerOff") {
         $msg = "The virtual machine did not power off: $msg";
      } elsif ($op eq "suspend") {
         $msg = "The virtual machine did not suspend: $msg";
      } elsif ($op eq "resume") {
         $msg = "The virtual machine did not resume: $msg";
      } elsif ($op eq "reset") {
         $msg = "The virtual machine did not reset: $msg";
      }

      $msg .= "More information may be available in the virtual machine " .
         "event log.";

      $msg = error($msg);
   }

   return html($msg . $data);
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my ($ctx, $op) = @_;

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("cmd/##/");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", $op);
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());

   return ($trnCtx, "${reqPath}op/$op/");
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $msg) : $msg;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   $str =~ s/(\r\n|\r|\n)/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";
   my $vmId = $q->param("vmid");

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var hash="$vmId";var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
