#!/usr/bin/perl -w

#############################################################
# Copyright (C) 1998-2002 VMware, Inc.
# All Rights Reserved
# $Id: SxOptions.pm,v 1.1.2.10 2002/06/19 20:11:11 ghutchins Exp $
#############################################################
#
# SxOptions.pm
#
# A script that presents the setup options available.
package VMware::Management::SxOptions;

use strict;
use VMware::ExtHelpers qw(&Warning
                          $gPanicCallback
			  &clearWarnings);
use Carp qw(confess);
$SIG{__DIE__} = \&confess;
use CGI;
use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Management::ConfigHTML;
use VMware::DOMAccess;
use VMware::Management::Util;
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();
use VMware::Management::Session ();


my $RCom = "request";
my $RCom_Save = "savesecurity";
my $encryptRUI =  "ruissl";
my $encryptMUI =  "muissl";
my $htmlTitle = "Security Configuration";
my $user;
my $productName;

my $URL_INDEX = "/security/index.pl";
my $URL_SELF = "/security";

#####################################################################
# HTML producing functions                                          #
#####################################################################


# This function displays a page to make the user wait until
# the httpd server has restarted.
sub toWaitingForServerHTML($$$$$) {
   my ($cgi, $nextPage, $title, $msg, $timeOut) = @_;

   my $html = qq{<div id="sxStatHdr" class="sctHdr">$/} .
      qq{<table border="0" cellpadding="0" cellspacing="0">$/} .
      qq{<tr>$/} .
      qq{<td width="100%">$title</td>$/} .
      qq{</tr>$/} .
      qq{</table>$/} .
      qq{</div>$/} .
      qq{<p>Please wait for the operation to complete. } .
      qq{$msg, click <a href="$nextPage">here</a> to continue.</p>$/};

   return qq(<HTML><HEAD><TITLE>Waiting for operation to complete</TITLE>) .
      qq(<META http-equiv="Refresh" content="$timeOut;url=$nextPage" /></HEAD>) .
      qq(<body>$html</BODY></HEAD>\n);
}

sub updateConfigVar($$$) {
   my ($cgi, $key, $options) = @_;

   $options->{$key} = ($cgi->param($key) eq "on") ?  "yes" : "no";
}


sub usingNTHost() {
   if (!defined($productName)) {
      my %credentials = &VMware::Management::Util::hash_credentials();
      my $server = &VMware::Control::Server::new(undef, 0,
                                                 $credentials{ "l" },
                                                 $credentials{ "m" });
      my ($server_ok, $err, $str) = VMware::Management::Util::connect_ctrl_obj($server);
      if ($server_ok) {
         $productName = VMware::Management::Util::getProductName($server);
      }
      if (defined($server)) {
         $server->disconnect();
      }
   }

   if ($productName != "GSX") {
      return 0;
   }

   my $in = VMware::DOMAccess->new("exec");
   $in->setValue("exec.op", "GSXHostInfo_Query");
   my $out = &VMware::Management::ConfigHTML::request($in);

   my $hostOSval = $out->getValue(".out.hostOS");
   my $retval = $out->getValue(".out.hostOSVersion");

   $in->dispose();
   $out->dispose();

   if ($hostOSval eq "MSWin32") {
      return (($retval lt 5) ? 1 : 0);
   } else {
      return 0;
   }
}

sub getResultsString($$) {
   my ($name, $value) = @_;
   my $s = "";

   if ($value eq "yes") {
      $s .= "<li>Enabling ";
   } else {
      $s .= "<li>Disabling ";
   }
   if ($name eq $encryptMUI) {
      $s .= "SSL for the management interface.</li>";
   } else {
      #$name eq $encryptRUI
      $s .= "SSL for the server console.</li>";
   }
}

sub error {
   return qq{<center>$/} .
      qq{<table width="475" border="0" cellpadding="5" cellspacing="0">$/} .
      qq{<tr>$/} .
      qq{<td width="32"><img src="/vmware/imx/rc-error.png" width="22" height="22"/></td>$/} .
      qq{<td><b>Error</b></td>$/} .
      qq{</tr>$/} .
      qq{</table>$/} .
      qq{<table width="475" border="0" cellpadding="5" cellspacing="0">$/} .
      qq{<tr>$/} .
      qq{<td colspan="2">$/} .
      qq{<p>One or more errors were encountered while applying your changes.</p>$/} .
      qq{$_[0]$/} .
      qq{</td>$/} .
      qq{</tr>$/} .
      qq{</table>$/} .
      qq{</center>$/};
}

#
# saveConfig()
#
# Make a serverd request to save the security settings,
# and activate the changes.
sub saveConfig($) {
    my ($cgi) = @_;
    my %options;
    my %results;
    my $timeOut;
    my $s = "";
    my $errorList = "";
    my $successList = "";
    my $key;

    if (!usingNTHost()) {
       updateConfigVar($cgi, $encryptMUI, \%options);
    }
    updateConfigVar($cgi, $encryptRUI, \%options);

    # Convert the options values to reasonable 1/0 settings.
    foreach $key (keys %options) {
       if ($options{$key} =~ m/Y|T|1/i) {
          $options{$key} = 1;
       } else {
          $options{$key} = 0;
       }
    }

    if (!activateChanges(\%options, \%results)) {
       $s .= error("<p>SSL Internal Error. Please file an incident report with VMware support.</p>\n");
       return $s;
    }

    # Go through the results of activateChanges and check that all calls to
    # enable/disable SSL were successful.  If a test fails, add it to a list
    # of calls which get printed out to an error page.
    foreach $key (keys %results) {
       if (!$results{$key}) {
         # Failed to enable SSL for this key.
         $errorList .= getResultsString($key, $options{$key});
       } else {
          # The call was successful.
          $successList .= getResultsString($key, $options{$key});
       }
    }

    if ($errorList ne "") {
       # An error occurred. Go to error page.
       my $errorMsg = "<p>The following operations failed: " .
                      "<ul>$errorList</ul></p>";
       if ($successList ne "") {
          # Also list the operations which succeeded.
          $errorMsg .= "<p>The following operations succeeded: " .
                       "<ul>$successList</ul></p>";
       }

       $s .= error($errorMsg);
       return $s;
    }

    my $nextUrl = $cgi->url();
    if ($^O eq "MSWin32") {
       if ($results{$encryptMUI}) {
          #kind of a hack, but I don't know how to do the neat server side
          #url rewriting that apache does.
          if ($options{$encryptMUI} =~ /(Y|T|1)/i) {
             $nextUrl =~ s/8222/8333/;
             $nextUrl =~ s/^http:/https:/;
          } else {
             $nextUrl =~ s/8333/8222/;
             $nextUrl =~ s/^https:/http:/;
          }
       }
       $timeOut = 1;
    } else {
       $timeOut = 12;  #must be long enough for apache to restart
    }

    $s .= toWaitingForServerHTML($cgi, $nextUrl,
                                 "Saving Changes and Restarting Server",
                                 "If this page does not automatically "
                                 . "reload within 15 seconds",
                                 $timeOut);
    return $s;
}


sub remoteUISecurity($){
   my ($checked) = @_;
   my $text = 
      qq{<tr class="cat">$/} .
      qq{<td colspan="2"><div>VMware Server Console</div></td>$/} .
      qq{</tr>$/} .
      qq{<tr>$/} .
      qq{<td class="val" nowrap="1">$/} .
      qq{<table class="cntnr" border="0" cellpadding="0" cellspacing="0">$/} .
      qq{<tr>$/} .
      qq{<td><input name="$encryptRUI" type="checkbox" $checked->{$encryptRUI}/></td>$/} .
      qq{<td>&nbsp;</td>$/} .
      qq{<td width="100%" nowrap="1">Use Secure Sockets Layer (SSL) to encrypt remote server console connections</td>$/} .
      qq{</tr>$/} .
      qq{</table>$/} .
      qq{</td>$/} .
      qq{</tr>$/};

   return $text;
}

sub managementUISecurity($){
   my ($checked) = @_;

   if (usingNTHost()) {
      return qq{<tr class="cat">$/} .
         qq{<td colspan="2"><div>Management Interface</div></td>$/} .
         qq{</tr>$/} .
         qq{<tr>$/} .
         qq{<td class="val" nowrap="1">$/} .
         qq{<div>You are using Windows NT as a host operating system.</div>$/} .
         qq{<div>If you would like to use Secure Sockets Layer (SSL) } .
         qq{to encrypt management interface sessions,</div>$/} .
         qq{<div>please follow the directions detailed in the documentation.</div>$/} .
         qq{</td>$/} .
         qq{</tr>$/};
   } else {
      my $encrypted = $checked->{$encryptMUI} ne "" ? 1 : 0;
	  my $text = 
	     qq{<tr class="cat">$/} .
         qq{<td colspan="2"><div>Management Interface</div></td>$/} .
         qq{</tr>$/} .
         qq{<tr>$/} .
         qq{<td class="val" nowrap="1">$/} .
         qq{<table class="cntnr" border="0" cellpadding="0" cellspacing="0">$/} .
         qq{<tr>$/} .
         qq{<td><input name="$encryptMUI" type="checkbox" $checked->{$encryptMUI}/><input name="alreadyIsEncrypted" type="hidden" value="$encryptMUI=$encrypted"/></td>$/} .
         qq{<td>&nbsp;</td>$/} .
         qq{<td width="100%" nowrap="1">Use Secure Sockets Layer (SSL) to encrypt management interface sessions</td>$/} .
         qq{</tr>$/} .
         qq{</table>$/} .
         qq{</td>$/} .
         qq{</tr>$/};

	  return $text;
   }
}

sub toCustomSecurityHTML($) {
    my ($cgi) = @_;
    my $url_self = $cgi->url(-relative=>1);

    my $s = "";

    $s .=
        ("<script type=\"text/javascript\">\n" .
	 "<!--\n" .
	 "function doConfirmSave() {\n" .
	 "   return confirm(\"Are you sure you want to change the security configuration?\");\n" .
	 "}\n" .
	 "function doSuccess() {\n" .
	 "   return confirm(\"success!\");\n" .
	 "}\n" .
	 "// -->\n" .
	 "</script>\n" .

	 "<form name=\"save\" method=\"POST\" action=\"$url_self\", onSubmit=\"return doConfirmSave()\" onError=\"return doSuccess()\">\n" .
         "\n" );

    my %checked;
    my %options;
    loadSecurityOptions(\%options, "host");

    foreach my $key ($encryptMUI, $encryptRUI) {
       if ($options{$key} =~ m/(Y|T|1)/i) {
         $checked{$key} = "CHECKED";
       } else {
         $checked{$key} = "";
       }
    }

    $s .= qq{<div id="sxStatHdr" class="sctHdr">$/} .
       qq{<table border="0" cellpadding="0" cellspacing="0">$/} .
       qq{<tr>$/} .
       qq{<td width="100%">Security Options</td>$/} .
       qq{</tr>$/} .
       qq{</table>$/} .
       qq{</div>$/} .
       qq{<div class="cfg">$/} .
       qq{<table border="0" cellpadding="0" cellspacing="0">$/};

    $s .= managementUISecurity(\%checked);
    $s .= remoteUISecurity(\%checked);

    $s .= qq{</table>$/} .
       qq{</div>$/};

    #XXX ugly
    my $nextUrl = $cgi->url();
    $nextUrl =~ s/$URL_SELF.*$/$URL_INDEX/;
    $s .= qq{$/} .
       qq{<div id="std">$/} .
       qq{<table border="0" cellpadding="0" cellspacing="0" style="border-top: 2px solid #E6E6E6; margin-top: 24px">$/} .
       qq{<tr>$/$/} .
       qq{<td width="100%">&nbsp;</td>$/} .
       qq{<td class="btnColLft">$/} .
       qq{<div class="btn" onclick="document.forms[0].submit();">$/} .
       qq{<table>$/} .
       qq{<tr>$/} .
       qq{<input type="hidden" name="$RCom" value="$RCom_Save">$/} .
       qq{<td class="lbl" nowrap="1">Save Changes</td>$/} .
       qq{</tr>$/} .
       qq{</table>$/} .
       qq{</div>$/} .
       qq{</td>$/} .
       qq{<td class="btnCol">$/} .
       qq{<div class="btn" onclick="document.forms[0].reset()">$/} .
       qq{<table>$/} .
       qq{<tr>$/} .
       qq{<td class="lbl" nowrap="1">Revert Changes</td>$/} .
       qq{</tr>$/} .
       qq{</table>$/} .
       qq{</div>$/} .
       qq{</td>$/} .
       qq{</tr>$/} .
       qq{</table>$/} .
       qq{</div>$/};

    return $s;
}


#
# loadSecurityOptions
#
# make a serverd request to load all of the security.$subfield entries
# from the host pref database.
#
sub loadSecurityOptions($$) {
   my ($optRef,$subfield) = @_;
   my $in = VMware::DOMAccess->new("exec");
   my $key;
   my $i = 0;

   $in->setValue("exec.op", "Security_GetConfig");
   $in->setValue("exec.in.pattern", "security.$subfield");
   my $out = &VMware::Management::ConfigHTML::request($in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return;
   }

   # XXX umm.  we lose all relevant error codes here
   while ($key = $out->getValue(".out.pref[$i].name")) {
      $optRef->{$key} = $out->getValue(".out.pref[$i].value");
      $i++;
   }

   my $ctx = &VMware::Management::Session::get_temp_context();

   # Get mui and rui settings from vmdb
   $ctx->SetCurrentPath("/host2/#_mui/");
   $optRef->{"muissl"} = $ctx->GetBool("setup/sslSettings/mui/enabled");
   $optRef->{"ruissl"} = $ctx->GetBool("setup/sslSettings/authd/enabled");

   $in->dispose();
   $out->dispose();
}


#
# setSSL --
#
# Issue a vmdb command to enable/disable ssl for the specified component.
#
sub setSSL($$) {
   my ($component, $value) = @_;
   my $ctx = &VMware::Management::Session::get_temp_context();

   $ctx->SetCurrentPath("/host2/#_mui/setup/sslSettings/");
   $ctx->BeginTransaction();
   my $cmdPath = $ctx->NewArrayIndex("cmd/##");
   $cmdPath = $ctx->GetAbsPath($cmdPath);
   $ctx->SetString("${cmdPath}op", "setSSLEnabled");
   $ctx->SetString("${cmdPath}user",
                   &VMware::Management::Session::get_usertoken());
   $ctx->SetString("${cmdPath}op/setSSLEnabled/in/component", $component);
   $ctx->SetString("${cmdPath}op/setSSLEnabled/in/enabled", $value);
   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($ctx, $cmdPath);

   if (!$ok) {
      VMware::Management::Util::trace("setSSL failed: $msg", "error");
   } else {
      VMware::Management::Util::trace("setSSL succeeded: $component, $value",
                                      "debug");
   }
   $ctx->LocalArrayUnset($cmdPath, 1);
   return $ok;
}


sub activateChanges($$) {
   my ($optRef, $results) = @_;
   my $in = VMware::DOMAccess->new("exec");
   my $key;
   my $i = 0;
   my $count = 0;

   $in->setValue("exec.op", "Security_ActivateConfig");
   foreach $key (keys %{$optRef}) {
      if ($key ne "muissl" && $key ne "ruissl") {
         $count++;
         $in->setValue("exec.in.pref[$i].name", $key);
         $in->setValue("exec.in.pref[$i].value", $optRef->{$key});
         $i++;
      }
   }

   if ($count > 0) {
      my $out = &VMware::Management::ConfigHTML::request($in);
      if (!$out) {
         Warning("Serverd request failed. Internal error.");
         $in->dispose();
         return 0;
       }

       $i = 0;
       while ($key = $out->getValue(".out.pref[$i].name")) {
          $results->{$key} = $out->getValue(".out.pref[$i].value");
          $i++;
       }

       $out->dispose();
   }

   my %optionsOrig;
   loadSecurityOptions(\%optionsOrig, "host");

   # Only do the call if there has been a change.
   if ($optionsOrig{"muissl"} != $optRef->{"muissl"}){
      $results->{"muissl"} = setSSL("mui", $optRef->{"muissl"});
   }

   if ($optionsOrig{"ruissl"} != $optRef->{"ruissl"}) {
      $results->{"ruissl"} = setSSL("authd", $optRef->{"ruissl"});
   }

   return 1;
   $in->dispose();
}


sub handleRequest($) {
    my ($cgi) = @_;
    my $s = "";
    my $url_self = $cgi->url(-relative=>1);
    my $title = "Customize Security Configuration";

    # XXX Hack
    $gPanicCallback = \&VMware::Management::ConfigHTML::PanicCallbackHTML;

    my $req = $cgi->param($RCom);

    if ( defined($req)) {
       if ($req eq $RCom_Save ) {
          return (saveConfig($cgi));
       }
    }

    $s = toCustomSecurityHTML($cgi);
    return ($s);
}

sub handler {
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'});

    umask 022;   # Create files that are not writable by group

    &clearWarnings();

    my $cgi = new CGI( shift( ) );
    my %MimeHeaders = ( "Content-Type"  => "text/html; charset=utf-8",
			"Pragma"        => "no-cache",
			"Cache-Control" => "no-cache",
			);

    my $head = qq{<html><head><title>VMware Management Interface</title>$/} .
       qq{<script src="/vmware/src/init.js"></script>$/} .
       qq{<script src="/vmware/src/xuaLib.js"></script>$/} .
       qq{<link rel="stylesheet" href="/vmware/css/usage.css" type="text/css">$/} .
       qq{<link rel="stylesheet" href="/vmware/css/page.css" type="text/css">$/} .
       qq{</head><body id="bodyObj">$/};

    my $s = "";

    if (!&VMware::Management::ConfigHTML::isAdminUser()) {
       my $url_self = $cgi->url(-relative=>1);

       $s = qq{<div id="sxStatHdr" class="sctHdr">$/} .
          qq{<table border="0" cellpadding="0" cellspacing="0">$/} .
          qq{<tr>$/} .
          qq{<td width="100%">Security Options</td>$/} .
          qq{</tr>$/} .
          qq{</table>$/} .
          qq{</div>$/} .
          qq{<div class="cfg">$/} .
          qq{<table border="0" cellpadding="0" cellspacing="0">$/} .
          qq{<tr>$/} .
          qq{<td class="val" nowrap="1">$/} .
          qq{<div>$/} .
          qq{<b>Error: Operation Not Permitted.</b>$/} .
          qq{<p>You must have administrative privileges to perform this operation.<\p>$/} .
          qq{</div>$/} .
          qq{</td>$/} .
          qq{</tr>$/};
          qq{</table>$/} .
          qq{</div>$/};

    } else {
       $s .= handleRequest($cgi);
    }

    my $tail = qq{</body></html>$/};

    $s = "$head$s$tail";

    return ($s, %MimeHeaders);
}

1;
