# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxRegisterVm;
# File: VMware/Management/SxRegisterVm.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();

use CGI ();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/");

   my $data = "";

   if (! $q->param("vmcfg")) {
      return html(error("Register/Unregister Virtual Machine: " .
                        "No virtual machine specified."));
   }

   if (! defined($q->param("vmid"))) {
      $q->param("vmid", isRegisteredVmCfg($ctx, $q->param("vmcfg")));
   }

   if ($q->param("op") =~ /^(register)$/i) {
      if (defined($q->param("vmid"))) {
         my $vmcfg = $q->param("vmcfg");
         $data .= warning("Register Virtual Machine: " .
                          "$vmcfg: Already registered.");
      } else {
         $data .= register($ctx);
      }
   }

   if ($q->param("op") =~ /^(unregister)$/i) {
      if (! defined($q->param("vmid"))) {
         my $vmcfg = $q->param("vmcfg");
         $data .= warning("Unregister Virtual Machine: " .
                          "$vmcfg: Not registered.");
      } else {
         $data .= unregister($ctx);
      }
   }

   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift() || "";

   my $vmcfg = $q->param("vmcfg");
   my $vmid = isRegisteredVmCfg($ctx, $vmcfg);
   my $registered = "false";
   my $dn = $vmcfg;

   if (defined($vmid)) {
      $dn = $ctx->GetString("/vm/#${vmid}/vmx/cfgState/val/name");
      $registered = "true";
      $vmid = qq{"$vmid"};
   } else {
      $vmid = "null";
   }

   $vmcfg = jsStringLiteral($vmcfg);
   $dn = jsStringLiteral($dn);

   $data .= "var data={};$/";
   $data .= qq{data.vmcfg=$vmcfg;$/};
   $data .= qq{data.vmid=$vmid;$/};
   $data .= qq{data.dn=$dn;$/};
   $data .= qq{data.registered=$registered;};

   return html($data);
}


# ---------------------------------------------------------------------------

sub isRegisteredVmCfg {
   my $ctx = shift();
   my $vmcfg = shift();

   my $path = "/host2/#_mui/vms/registeredvms/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      if ($ctx->GetString("${path}canonicalCfgPath") eq $vmcfg) {
         my $vmid = $ctx->GetString("${path}vm");
         $vmid =~ s{.*\#(\w+)/?$}{$1};
         return $vmid;
      }
   }

   return undef;
}


# ---------------------------------------------------------------------------

sub register {
   my $ctx = shift();
   my %credentials = VMware::Management::Util::hash_credentials();

   my ($trnCtx, $reqPath) = cmdHead($ctx);

   $trnCtx->SetString("${reqPath}op", "register");
   $reqPath = $trnCtx->GetAbsPath("${reqPath}op/register");

   $trnCtx->SetString("${reqPath}in/cfgPath", $q->param("vmcfg"));
   $trnCtx->SetString("${reqPath}in/username", $credentials{ "l" });
   $trnCtx->SetBool("${reqPath}in/inVmList", 1);
   
   my ($ok, $data) = cmdTail($trnCtx, $reqPath);

   return $data;
}


# ---------------------------------------------------------------------------

sub unregister {
   my $ctx = shift();
   my %credentials = VMware::Management::Util::hash_credentials();

   my ($trnCtx, $reqPath) = cmdHead($ctx);

   $trnCtx->SetString("${reqPath}op", "unregister");
   $reqPath = $trnCtx->GetAbsPath("${reqPath}op/unregister");

   $trnCtx->SetString("${reqPath}in/cfgPath", $q->param("vmcfg"));
   $trnCtx->SetString("${reqPath}in/username", $credentials{ "l" });
   $trnCtx->SetBool("${reqPath}in/inVmList", 1);

   my ($ok, $data) = cmdTail($trnCtx, $reqPath);

   return $data;
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();
   my $op = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/vms/cmd/##");
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());

   return ($trnCtx, $trnCtx->GetAbsPath($reqPath));
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
      $data .= error("Register/Unregister Virtual Machine: $msg.");
   } else {
      my $vmid = $trnCtx->GetString("${reqPath}out/vmListPath/");
      if ($vmid) {
         $vmid = $trnCtx->GetString("${vmid}vm");
         $vmid =~ s/.*#(\w+)$/$1/;
         $q->param("vmid", $vmid);
      } else {
         $q->param("vmid", undef);
      }
   }

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   VMware::Management::Util::trace("$html", "debug");

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
