# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxSnmpConfiguration;
# File: VMware/Management/SxSnmpConfiguration.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();
use VMware::Management::Session ();

use CGI ();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $data = "";

   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/host2/#_mui/snmpconfig");

   my ($ok, $msg);
   for ($q->param()) {
      VMware::Management::Util::trace("PARAM: $_ --> " . $q->param($_),
                                      "debug");

      if (/^masterstatus$/i) {
         ($ok, $msg) = set($ctx, lc($_), "bool", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change master " .
                           "agent status: $msg");
         }
      }

      if (/^runatsystemstartup$/i) {
         ($ok, $msg) = set($ctx, lc($_), "bool", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change master " .
                           "agent startup type: $msg");
         }
      }

      if (/^startupscript$/i) {
         ($ok, $msg) = set($ctx, lc($_), "str", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change master " .
                           "agent startup script location: $msg");
         }
      }

      if (/^configfile$/i) {
         ($ok, $msg) = set($ctx, lc($_), "str", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change master " .
                           "agent configuration file location: $msg");
         }
      }

      if (/^runsubagent$/i) {
         ($ok, $msg) = set($ctx, lc($_), "bool", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change VMware " .
                           "subagent status: $msg");
         }
      }

      if (/^vmwaretraps$/i) {
         ($ok, $msg) = set($ctx, lc($_), "bool", $q->param($_));
         if (! $ok) {
            $data .= error("SNMP Configuration: Could not change VMware " .
                           "subagent traps: $msg");
         }
      }
   }

   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift() || "";
   my ($ok, $ret);

   $data .= "var data={};$/";
   $data .= "data.agent={};$/";

   # Master SNMP Agent Status
   ($ok, $ret) = get($ctx, "masterstatus", "bool");
   if ($ok) {
      $data .= "data.agent.running=";
      $data .= $ret ? "true" : "false";
      $data .= ";$/";
   } else {
      $data .= error("SNMP Configuration: Error while retrieving master " .
                     "agent status: $ret");
   }

   # Master SNMP Agent Startup Type
   ($ok, $ret) = get($ctx, "runatsystemstartup", "bool");
   if ($ok) {
      $data .= "data.agent.autoStart=";
      $data .= $ret ? "true" : "false";
      $data .= ";$/";

      VMware::Management::Util::trace("data.agent.autoStart=" .
                                      ($ret ? "true" : "false") . ";",
                                      "debug");
   } else {
      $data .= error("SNMP Configuration: Error while retrieving master " .
                     "agent startup type: $ret");
   }

   # Master SNMP Agent Startup Script
   ($ok, $ret) = get($ctx, "startupscript", "str");
   if ($ok) {
      $data .= qq{data.agent.scriptLoc="$ret";$/};
   } else {
      $data .= error("SNMP Configuration: Error while retrieving master " .
                     "agent startup script location: $ret");
   }

   # Master SNMP Agent Configuration File
   ($ok, $ret) = get($ctx, "configfile", "str");
   if ($ok) {
      $data .= qq{data.agent.configLoc="$ret";$/};
   } else {
      $data .= error("SNMP Configuration: Error while retrieving master " .
                     "agent configuration file location: $ret");
   }

   $data .= "data.subagent={};";
   $data .= "data.subagent.traps={};$/";

   # VMware SNMP SubAgent Status
   ($ok, $ret) = get($ctx, "runsubagent", "bool");
   if ($ok) {
      $data .= "data.subagent.enabled=";
      $data .= $ret ? "true" : "false";
      $data .= ";$/";
   } else {
         $data .= error("SNMP Configuration: Error while retrieving VMware " .
                        "subagent status: $ret");
   }

   # VMware SNMP SubAgent VMware Traps
   ($ok, $ret) = get($ctx, "vmwaretraps", "bool");
   if ($ok) {
      $data .= "data.subagent.traps.enabled=";
      $data .= $ret ? "true" : "false";
      $data .= ";$/";
   } else {
         $data .= error("SNMP Configuration: Error while retrieving VMware " .
                        "subagent traps: $ret");
   }

   return html($data);
}


# ---------------------------------------------------------------------------

sub get {
   my $ctx = shift();
   my $cmd = shift();
   my $type = shift() || "str";

   my ($trnCtx, $reqPath) = cmdHead($ctx, $cmd);

   return cmdTail($trnCtx, $reqPath, $type);
}


# ---------------------------------------------------------------------------

sub set {
   my $ctx = shift();
   my $cmd = shift();
   my $type = shift() || "str";
   my $val = shift();

   my ($trnCtx, $reqPath) = cmdHead($ctx, $cmd);

   if ($type eq "bool") {
      VMware::Management::Util::trace($trnCtx->SetBool("${reqPath}in/value",
                                                       $val),
                                      "debug");

   } elsif ($type eq "int") {
      VMware::Management::Util::trace($trnCtx->SetInt("${reqPath}in/value",
                                                       $val),
                                      "debug");
   } else {
      VMware::Management::Util::trace($trnCtx->SetString("${reqPath}in/value",
                                                       $val),
                                      "debug");
   }

   return cmdTail($trnCtx, $reqPath, $type);
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();
   my $cmd = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("cmd/##");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", "$cmd");
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());

   return ($trnCtx, "${reqPath}op/$cmd/");
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $type = shift();

   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
      $data = $msg;
   } else {
      if ($type eq "bool") {
         $data = $trnCtx->GetBool("${reqPath}out/value");
      } elsif ($type eq "int") {
         $data = $trnCtx->GetInt("${reqPath}out/value");
      } else {
         $data = $trnCtx->GetString("${reqPath}out/value");
      }
   }

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   $str =~ s/(\r\n|\r|\n)/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
