# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxUsersAndGroups;
# File: VMware/Management/SxUsersAndGroups.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::Session ();

use CGI ();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/host2/#_mui/users");

   if ($q->param("op") =~ /^addUsr$/i) {
      return addUsr($ctx);
   }

   if ($q->param("op") =~ /^setUsr$/i) {
      return setUsr($ctx);
   }

   if ($q->param("op") =~ /^remUsr$/i) {
      return remUsr($ctx);
   }

   if ($q->param("op") =~ /^addGrp$/i) {
      return addGrp($ctx);
   }

   if ($q->param("op") =~ /^setGrp$/i) {
      return setGrp($ctx);
   }

   if ($q->param("op") =~ /^remGrp$/i) {
      return remGrp($ctx);
   }

   my @data = data($ctx);

   return @data;
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift() || "";

   $data .= "function Usr(nm,hd,grp,grps){" .
      "this.nm=nm;" .       # Name
      "this.hd=hd;" .       # Home directory
      "this.grp=grp;" .     # Primary group
      "this.grps=grps;}$/"; # Groups

   $data .= "function Grp(nm,usrs){" .
      "this.nm=nm;" .       # Name
      "this.usrs=usrs;}$/"; # Users

   # (<usr> => (hd => <homedir>, grp => <grp>, grps => [<grp>, ...]))
   my %usrs = usrData($ctx);
   # (<grp> => [<usr>, ...])
   my %grps = grpData($ctx, \%usrs);

   my $usrStr = "";
   for (sort(keys(%usrs))) {
      $usrStr .= ($usrStr ? ",$/" : "") . "new Usr(";
      $usrStr .= qq{"$_","$usrs{$_}{hd}","$usrs{$_}{grp}",};
      if (scalar(@{$usrs{$_}{grps}})) {
         $usrStr .= '["' . join('","', sort(@{$usrs{$_}{grps}})) . '"])';
      } else {
         $usrStr .= '[])';
      }
   }
   $data .= qq{var usrs=[$usrStr];$/};

   my $grpStr = "";
   for (sort(keys(%grps))) {
      $grpStr .= ($grpStr ? ",$/" : "") . "new Grp(";
      $grpStr .= qq{"$_",};
      if (scalar(@{$grps{$_}})) {
         $grpStr .= '["' . join('","', sort(@{$grps{$_}})) . '"])';
      } else {
         $grpStr .= '[])';
      }
   }
   $data .= qq{var grps=[$grpStr];};

   return html($data);
}


# ---------------------------------------------------------------------------

sub usrData {
   my $ctx = shift();
   my %usrs;

   my $path = "userlist/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $name = $ctx->GetString("${path}name");
      my $grp  = $ctx->GetString("${path}defaultgroup") || "";
      my $home = $ctx->GetString("${path}homedir");

      $usrs{$name} = {"hd" => $home, "grp" => $grp, "grps" => []};
   }

   return %usrs;
}


# ---------------------------------------------------------------------------

sub grpData {
   my $ctx = shift();
   my $usrs = shift();
   my %grps;

   my $path = "grouplist/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $name = $ctx->GetString("${path}name");
      $grps{$name} = [];

      my $list = "${path}memberlist/#";
      while (defined($list = $ctx->GetNextSibling($list))) {
         my $usr = $ctx->GetString("${list}member");
         if (defined($usrs->{$usr}) && $name ne $usrs->{$usr}{grp}) {
            push (@{$usrs->{$usr}{grps}}, $name);
         }

         push(@{$grps{$name}}, $usr);
      }
   }

   return %grps;
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my $op = shift();
   VMware::Management::Util::trace("cmdHead: $op [", "debug");

   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath("/host2/#_mui/users");

   $trnCtx->BeginTransaction();

   my $cmdPath = $trnCtx->NewArrayIndex("cmd/##");
   $cmdPath = $trnCtx->GetAbsPath($cmdPath);
   $trnCtx->SetString("${cmdPath}op", $op);


   VMware::Management::Util::trace("] cmdHead: $op", "debug");
   return ($trnCtx, "${cmdPath}op/${op}/");
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $cmdPath = shift();

   $cmdPath =~ s{^(.*)(cmd/\#\#.*?/).*}{$1$2};
   VMware::Management::Util::trace("cmdTail: $cmdPath [", "debug");

   if ($trnCtx->EndTransaction(1) != 0) {
      my $msg = error("Transaction failed.");
      return wantarray ? (0, $msg) : $msg;
   }

   my $status;
   for (1..30) {
      sleep(1);

      $status = $trnCtx->GetString("${cmdPath}status");

      if ($status =~ /^(done|error)$/) {
         last;
      }
   }

   if ($status eq "error") {
      my $msg = error($trnCtx->GetString("${cmdPath}status/error/errMsg"));
      return wantarray ? (0, $msg) : $msg;
   }

   VMware::Management::Util::trace("] cmdTail: $cmdPath", "debug");
   return wantarray ? (1, "") : "";
}


# ---------------------------------------------------------------------------

sub addUsr {
   my $ctx = shift();
   my $name = $q->param("nm");
   VMware::Management::Util::trace("NAME: $name", "debug");
   my $password = $q->param("pw");
   my $home = $q->param("hd");
   $home ||= "/home/$name";
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   ($trnCtx, $cmdPath) = cmdHead("adduser");

   $trnCtx->SetString("${cmdPath}in/username", $name);
   $trnCtx->SetString("${cmdPath}in/password", $password);
   $trnCtx->SetString("${cmdPath}in/homedir", $home);

   ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
   if (! $ok) {
      $data .= $msg;
      return data($ctx, $data);
   }

   # Add groups.
   my @addGroups = split(",", $q->param("ag"));
   for my $grp (@addGroups) {
      ($trnCtx, $cmdPath) = cmdHead("addusertogroup");

      $trnCtx->SetString("${cmdPath}in/username", $name);
      $trnCtx->SetString("${cmdPath}in/newgroup", $grp);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub setUsr {
   my $ctx = shift();
   my $name = $q->param("nm");
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   # Change user home directory and password.
   ($trnCtx, $cmdPath) = cmdHead("chuser");

   my $home = $q->param("hd");
   my $password = $q->param("pw");

   $trnCtx->SetString("${cmdPath}in/username", $name);
   $trnCtx->SetString("${cmdPath}in/newhomedir", $home);
   if (defined($password) && $password ne "") {
      $trnCtx->SetString("${cmdPath}in/newpassword", $password);
   }

   ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
   $data .= $msg;

   # Add groups.
   my @addGroups = split(",", $q->param("ag"));
   for my $grp (@addGroups) {
      VMware::Management::Util::trace("ADD GRP: $grp", "debug");
      ($trnCtx, $cmdPath) = cmdHead("addusertogroup");

      $trnCtx->SetString("${cmdPath}in/username", $name);
      $trnCtx->SetString("${cmdPath}in/newgroup", $grp);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # Remove groups.
   my @remGroups = split(",", $q->param("rg"));
   for my $grp (@remGroups) {
      VMware::Management::Util::trace("REM GRP: $grp", "debug");
      ($trnCtx, $cmdPath) = cmdHead("deluserfromgroup");

      $trnCtx->SetString("${cmdPath}in/username", $name);
      $trnCtx->SetString("${cmdPath}in/oldgroup", $grp);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub remUsr {
   my $ctx = shift();
   my $name = $q->param("nm");
   VMware::Management::Util::trace("NAME: $name", "debug");
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   ($trnCtx, $cmdPath) = cmdHead("removeuser");

   $trnCtx->SetString("${cmdPath}in/username", $name);

   ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
   $data .= $msg;

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub addGrp {
   my $ctx = shift();
   my $name = $q->param("nm");
   VMware::Management::Util::trace("NAME: $name", "debug");
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   ($trnCtx, $cmdPath) = cmdHead("addgroup");

   $trnCtx->SetString("${cmdPath}in/groupname", $name);

   ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
   if (! $ok) {
      $data .= $msg;
      return data($ctx, $data);
   }

   # Add users.
   my @addUsers = split(",", $q->param("au"));
   for my $usr (@addUsers) {
      ($trnCtx, $cmdPath) = cmdHead("addusertogroup");

      $trnCtx->SetString("${cmdPath}in/username", $usr);
      $trnCtx->SetString("${cmdPath}in/newgroup", $name);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub setGrp {
   my $ctx = shift();
   my $name = $q->param("nm");
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   # Add users.
   my @addUsers = split(",", $q->param("au"));
   for my $usr (@addUsers) {
      ($trnCtx, $cmdPath) = cmdHead("addusertogroup");

      $trnCtx->SetString("${cmdPath}in/username", $usr);
      $trnCtx->SetString("${cmdPath}in/newgroup", $name);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # Remove users.
   my @remUsers = split(",", $q->param("ru"));
   for my $usr (@remUsers) {
      ($trnCtx, $cmdPath) = cmdHead("deluserfromgroup");

      $trnCtx->SetString("${cmdPath}in/username", $usr);
      $trnCtx->SetString("${cmdPath}in/oldgroup", $name);

      ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
      $data .= $msg;
   }

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub remGrp {
   my $ctx = shift();
   my $name = $q->param("nm");
   VMware::Management::Util::trace("NAME: $name", "debug");
   my $data = "";
   my ($trnCtx, $cmdPath, $ok, $msg);

   ($trnCtx, $cmdPath) = cmdHead("removegroup");

   $trnCtx->SetString("${cmdPath}in/groupname", $name);

   ($ok, $msg) = cmdTail($trnCtx, $cmdPath);
   $data .= $msg;

   # XXX: It's possible that data will return before the new info is published.
   sleep(3);
   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
