# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::VmConfiguration;
# File: VMware/Management/VmConfiguration.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();
use VMware::Management::Session ();
use VMware::Management::SS::VMUtil ();
use VMware::HConfig::SSHostRemote ();
use VMware::HConfig::MFSFileSystemRemote ();
use VMware::HConfig::RawDisk ();
use VMware::Management::SS::StorageConfiguration ();
use VMware::Management::SS::NetworkConnections ();
use VMware::HConfig::Constant qw($gVMFSMaxKBlocks $gVMFS2MaxKBlocks
                                 $gVMFSBlockSizeLimit $gVMFS2BlockSizeLimit
                                 $gVMFSAbsMaxFileSize $gVMFS2AbsMaxFileSize);
use VMware::Management::Util;

use CGI ();
my $q;

sub HOSTIF            () { "hi" }     # hostif as in file or device
sub START_CONNECTED   () { "rdsc" }   # startConnected as for removable devices
sub CONNECT_STATE     () { "rdcs" }   # connectState as for removable devices
sub BIDIRECTIONAL     () { "bidi" }   # parallel bidirectional connection
sub DEPTH             () { "depth" }  # depth as for video
sub DEV_NAME          () { "devnm" }  # Device name as for nic (e.g., vmnic0)
sub EMULATION         () { "vdev" }   # emulation as for nic
sub VMNET             () { "vmnet" }  # custom NIC type
sub IDE_ID            () { "ideID" }  # IDE ID for a virtual disk
sub LOCATION          () { "loc" }    # filePath or hostdev
sub MODE              () { "mode" }   # mode as for scsiDev:disk
sub SIZE              () { "size" }   # size of the disk (GSX)
sub CLIENT_DEVICE     () { "clientDvc" } # CDROM cleint device
sub NETWORK_NAME      () { "netnm" }  # Device name as for nic (e.g., vmnic0)
sub NUM               () { "num" }    # Number of VCPUs
sub SCSI_ID           () { "scsiID" } # SCSI ID for a virtual disk
sub SHARED_BUS        () { "shbus" }  # bus sharing for SCSI controller
sub SIZE_MB           () { "mb" }     # sizeMb as for memory
sub END_POINT         () { "endp" }   # endPoint as in serial client or server
sub TRY_NO_RX_LOSS    () { "tnrl" }   # tryNoRxLoss serial connection to an app
sub YIELD_CPU_ON_POLL () { "poll" }   # yieldCpuOnPoll as opposed to interrupt
sub AUTO_CONNECT      () { "autoc" }  # auto connect USB devices

# Todo (get):
#
#  [x] SCSI Controller
#  [x] Hard Disk
#  [x] DVD/CD-ROM Drive
#  [x] Floppy Drive
#  [x] Network Adapter
#  [x] Serial Port   (?)
#  [x] Parallel Port (?)
#  [x] Generic SCSI  (?)
#  [x] Display Adapater
#
# Todo (set):
#
#  [x] SCSI Controller
#  [ ] Hard Disk
#  [ ] DVD/CD-ROM Drive
#  [x] Floppy Drive
#  [x] Network Adapter
#  [ ] Serial Port   (?)
#  [ ] Parallel Port (?)
#  [ ] Generic SCSI  (?)
#  [x] Display Adapater
#
# Todo (add):
#
#  [x] SCSI Controller
#  [ ] Hard Disk
#  [ ] DVD/CD-ROM Drive
#  [x] Floppy Drive
#  [ ] Network Adapter
#  [ ] Serial Port   (?)
#  [ ] Parallel Port (?)
#  [ ] Generic SCSI  (?)
#  [ ] Display Adapater
#
# Todo (rem):
#
#  [x] SCSI Controller
#  [x] Hard Disk
#  [x] DVD/CD-ROM Drive
#  [x] Floppy Drive
#  [x] Network Adapter
#  [x] Serial Port   (?)
#  [x] Parallel Port (?)
#  [x] Generic SCSI  (?)
#  [ ] Display Adapater (?)

# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/");

   for ($q->param()) {
      VMware::Management::Util::trace("OPT $_: " . $q->param($_));
   }

   if ($q->param("op") =~ /^getVmNetDevData$/i) {
      return html(vmNetDevData());
   }

   if ($q->param("op") =~ /^getVmDiskDevData$/i) {
      return html(vmDiskDevData($ctx));
   }

   if ($q->param("op") =~ /^getVmGenericScsiDevData$/i) {
      return html(vmGenericScsiDevData());
   }
   
   my $vmid = $q->param("vmid");
   if (defined($vmid) && $vmid ne "") {
      if (isRegisteredVmId($ctx, $vmid)) {
         $ctx->SetCurrentPath("/vm/#$vmid");
      } else {
         return html(error("Virtual machine not found: #$vmid"));
      }
   } else {
      return html(error("Missing virtual machine ID argument."));
   }

   if ($q->param("op") =~ /^(add)$/i) {
      if ($ctx->GetString("vmx/execState/val") ne "poweredOff") {
         return html(error("Add Device: Devices cannot be added unless " .
                           "the virtual machine is powered off."));
      }
      return add($ctx);
   }

   if ($q->param("op") =~ /^(rem)$/i) {
      if ($ctx->GetString("vmx/execState/val") ne "poweredOff") {
         return html(error("Remove Device: Devices cannot be removed unless " .
                           "the virtual machine is powered off."));
      }

      return rem($ctx);
   }

   if ($q->param("op") =~ /^(upgrade)$/i) {
      if ($ctx->GetString("vmx/execState/val") ne "poweredOff") {
         return html(error("Upgrade Virtual Hardware: Upgrade cannot be " .
                           "performed unless the virtual machine is " .
                           "powered off."));
      }

      return upgrade($ctx);
   }

   if ($q->param("op") =~ /^checkDisk$/i) {
      return checkDisk($ctx);
   }

   return data($ctx);
}


# ---------------------------------------------------------------------------

sub checkDisk {
   my $ctx = shift();
   my $loc = $q->param("loc");

   my %props;
   my $data;
   my $devData;

   $props{&LOCATION} = jsStringNoQuote($loc);
   $props{&SIZE} = getDiskSize($ctx, $loc);

   $devData = devData(%props);

   $data = qq{var fromCheckDisk = true; var devs = [$devData];};

   return html($data);
}


# ---------------------------------------------------------------------------

sub isRegisteredVmId {
   my $ctx = shift();
   my $vmid = shift();

   my $path = "/host2/#_mui/vms/registeredvms/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      if ($ctx->GetString("${path}vm") =~ /#\Q$vmid\E/) {
         return 1;
      }
   }

   return 0;
}


# ---------------------------------------------------------------------------

sub devices {
   my $ctx = shift();
   my $dev = caller() eq __PACKAGE__ ? $q->param("dev") : undef;
   my @devices;

   if (defined($dev)) {
      @devices = split(",", $dev);
   } else {
      my $path = "vmx/cfgState/val/dev/#";
      while (defined($path = $ctx->GetNextSibling($path))) {
         if ($path !~ m{^.*/#_(RTC|audio)/?$}) {
            if ($path =~ m{^.*/#_(.+?)/?$}) {
               push(@devices, $1);
            }
         }
      }
   }

   return sort(@devices);
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift();
   my $msg = "";

   my @devices = devices($ctx);
   my @options;

   my ($trnCtx, $reqPath);
   if ($q->param("op") =~ /^set$/i) {
      ($trnCtx, $reqPath) = cmdHead($ctx);
      $trnCtx->SetCurrentPath($reqPath);
   } else {
      my @candidateDevices = grep($_ =~ /^(ide|scsi)Dev\d+:\d+$/i, @devices);
      
      my $genericScsiDev = 0;
      my $diskDev = 0;
      
      # Loop over the supplied devices looking for types of SCSI devices.
      while (@candidateDevices && (! $genericScsiDev || ! $diskDev)) {
	 my $dev = pop(@candidateDevices);
	    
	 if (! $genericScsiDev && getDevSubClass($ctx, $dev) eq "passthru") {
	    $genericScsiDev = 1;
	 } elsif (! $diskDev && getDevSubClass($ctx, $dev) eq "disk") {
	    $diskDev = 1;
	 }
      }
      
      if ($genericScsiDev) {
         $data .= vmGenericScsiDevData();
      }

      if ($diskDev) {
         $data .= vmDiskDevData($ctx);
      }

      if (grep($_ =~ /^nic.*$/i, @devices)) {
         $data .= vmNetDevData();
      }

      if (grep($_ =~ /^memory$/i, @devices)) {
         $data .= vmMemoryData($ctx);
      }
   }

   my $devData = "";
   for (@devices) {
      $devData .= ($devData ? ",$/" : "");

      if (/^serial.*$/i) {
         $devData .= devData(serial($ctx, $_, $trnCtx));
         next;
      }

      if (/^parallel.*$/i) {
         $devData .= devData(parallel($ctx, $_, $trnCtx));
         next;
      }

      if (/^usbCtlr.*$/i) {
         $devData .= devData(usbCtlr($ctx, $_, $trnCtx));
         next;
      }

      if (/^floppy.*$/i) {
         $devData .= devData(floppy($ctx, $_, $trnCtx));
         next;
      }

      if (/^memory.*$/i) {
         $devData .= devData(memory($ctx, $_, $trnCtx));
         next;
      }

      if (/^scsiCtlr\d+$/i) {
         $devData .= devData(scsiCtlr($ctx, $_, $trnCtx));
         next;
      }

      if (/^(ide|scsi)Dev\d+:\d+$/i) {
         $devData .= devData(storageDev($ctx, $_, $trnCtx));
         next;
      }

      if (/^nic.*$/i) {
         $devData .= devData(nic($ctx, $_, $trnCtx));
         next;
      }

      if (/^video.*$/i) {
         $devData .= devData(video($ctx, $_, $trnCtx));
         next;
      }

      # Remove ",$/".
      chomp($devData);
      chop($devData);
   }

   if ($q->param("op") =~ /^set$/i) {
      my ($ok, $err) = cmdTail($trnCtx, $reqPath);
      if (! $ok) {
         $err =~ s{\.?$/*$}{}s;
         $msg .= error("Edit Device: $err. Your changes were not saved.");
      }
   }

   updateHtSharing($ctx);

   $data .= qq{var devs = [$devData];};
   $data .= $msg;

   VMware::Management::Util::trace("DATA : $data", "debug");

   return html($data);
}


# ---------------------------------------------------------------------------
#
# sortByNetworkType--
#
#      Compare network device types ensuring that bonds come before vmnics and
#      that vmnics come before vmnets.
#
# ---------------------------------------------------------------------------

sub sortByNetworkType($$) {
   if ($a =~ /^bond/ && $b =~ /^(vmnic|vmnet)/) {
      return -1;
   }

   if ($a =~ /^vmnic/ && $b =~ /^vmnet/) {
      return -1;
   }

   if ($a =~ /^vmnic/ && $b =~ /^bond/) {
      return 1;
   }


   if ($a =~ /^vmnet/ && $b =~ /^(bond|vmnic)/) {
      return 1;
   }

   return $a cmp $b;
}


# ---------------------------------------------------------------------------

sub vmNetDevData {
   my $ctx = &VMware::Management::Session::get_temp_context();
   my $data = "";

   my %nicNames;
   #    my @vmnics = map {$_->{"team"} eq "none" ? $_->{"vnm"} : $_->{"team"}} vmnics($host);
   #    push(@vmnics, map {$_} @{$host->getVMNetBondAdapters()});
   #    my %nicNames = map {$_ => ""} @vmnics;

   my $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $net = $ctx->GetString("${path}network");

      $nicNames{$dev} = $net;
      VMware::Management::Util::trace("Device: $dev <-> Network: $net", "debug");
   }

   my $nicData = "";
   for (sort sortByNetworkType keys(%nicNames)) {
      my $net = jsStringLiteral($nicNames{$_});
      $nicData .= ($nicData ne "" ? "," : "");
      $nicData .= qq{"$_":$net};
   }
   $data .= "var nicNames={$nicData};$/";

   VMware::Management::Util::trace("$data", "debug");

   return $data;
}


# ---------------------------------------------------------------------------

sub vmDiskDevData {
   my $ctx = shift();
   my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id");
   my $diskData = "";
   my $data = "";

   if ($prdId eq "ESX") {
      my %credentials = VMware::Management::Util::hash_credentials();
      my $user = $credentials{"l"};
      my $pass = $credentials{"m"};
      my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

      if (! defined($host)) {
         return html(error("Could not create remote host object."));
      }

      if (! $host->isVMKernel()) {
         return html(error("Could not determine storage configuration: " .
                           "The VMkernel is not loaded."));
      }

      # Multiply bs by bsm to get max file size in GB.
      $data .= "var bsm={vmfs1:$gVMFSMaxKBlocks,vmfs2:$gVMFS2MaxKBlocks};$/";
      $data .= "var bsl={vmfs1:$gVMFSBlockSizeLimit,vmfs2:$gVMFS2BlockSizeLimit};$/";
      $data .= "var ams={vmfs1:$gVMFSAbsMaxFileSize,vmfs2:$gVMFS2AbsMaxFileSize};$/";

      $data .= "function FsVol(id,vl,mb,fmb,bs,ca,fl){" .
      "this.id=id;" .       # vmhbaX:X:X:X
      "this.vl=vl;" .       # Volume label
      "this.mb=mb;" .       # Capacity in megabytes
      "this.fmb=fmb;" .     # Free space in megabytes
      "this.bs=bs;" .       # block size
      "this.ca=ca;" .       # Cluster access
      "this.fl=fl;}$/";     # List of files on disk

      $data .= "function File(fn,mb,rwx,uid,gid,att,lmod){" .
      "this.fn=fn;" .       # File name
      "this.mb=mb;" .       # File size in megabytes
      "this.rwx=rwx;" .     # Permissions as a string (e.g., /[rwx-]+/)
      "this.uid=uid;" .     # User ID
      "this.gid=gid;" .     # Group ID
      "this.att=att;" .     # Attribute (e.g., cow)
      "this.lmod=lmod;}$/"; # Last modified string (e.g., "Jan 23 14:43")

      my $fsData = "";
      for (@{$host->getAllMFS()}) {
       VMware::Management::Util::trace($_->getVmkName(), "debug");
         $fsData .= ($fsData ? ",$/" : "") . fsData($_);
      }

      $data .= "var vols=[$fsData];$/";

      $data .= "function Dsk(id,c,h,s,mb,fs,pt){" .
      "this.id=id;" .         # VMkernel logical name (e.g., vmhba0:0:0)
      "this.c=c;" .           # Cylinders
      "this.h=h;" .           # Heads
      "this.s=s;" .           # Sectors
      "this.mb=mb;" .         # Capacity in megabytes
      "this.fs=fs;" .         # Unpartitioned space
      "this.pt=pt;}$/";       # Partition table as an array of partitions

      $data .= "function Prt(id,sc,ec,mb,tc,tn,fs,bs,vl,ca,sp){" .
      "this.id=id;" .         # Partition number
      "this.sc=sc;" .         # Logical start cylinder
      "this.ec=ec;" .         # Logical end cylinder
      "this.mb=mb;" .         # Capacity in megabytes
      "this.tc=tc;" .         # Partition type code (extended, Dump, VMFS)
      "this.tn=tn;" .         # Partition type name (extended, Dump, VMFS)
      "this.fs=fs;" .         # If formatted
      "this.bs=bs;" .         # Block size (max file size)
      "this.vl=vl;" .         # Volume label
      "this.ca=ca;" .         # Cluster access
      "this.sp=sp;}$/";       # Spanned partitions as an array of Prt objects

      $data .= "var dsks=[];$/";

      for my $disk (@{$host->getVMSCSIDisks()}) {
         if (defined($disk)) {
            $diskData .= ($diskData ? "$/" : "") .
             VMware::Management::SS::StorageConfiguration::diskData($host,
                                                                    $disk);
         }
      }
   } else {
      $data .= "var dsks=[];$/";
   }

   $data .= $diskData;

   VMware::Management::Util::trace("$data", "debug");

   return $data;
}


# ---------------------------------------------------------------------------

sub vmGenericScsiDevData {
   VMware::Management::Util::trace("IN vmGenericScsiDevData [", "debug");
   my $ctx = &VMware::Management::Session::get_temp_context();
   my $data = "var genScsi={};$/";

   if ($ctx->GetString("/host2/#_mui/productinfo/id") ne "ESX") {
      return $data;
   }

   $data .= "function genScsiDev(n,t){this.id=n;this.type=t;}$/";

   my $path = "/vmkernel/config/storage/tape/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $id = $ctx->GetString("${path}name");
      $data .= qq{genScsi["$id"]=new genScsiDev("$id","tape");$/};
   }

   my $path = "/vmkernel/config/storage/cdrom/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $id = $ctx->GetString("${path}name");
      my $dev .= qq{genScsi["$id"]=new genScsiDev("$id","cdrom");$/};
      $data .= $dev;
   }

   VMware::Management::Util::trace("] vmGenericScsiDevData OUT", "debug");
   return $data;
}


# ---------------------------------------------------------------------------

sub vmMemoryData {
   my $ctx = shift();
   my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id");

   if ($prdId eq "ESX") {
      my %credentials = VMware::Management::Util::hash_credentials();
      my $user = $credentials{"l"};
      my $pass = $credentials{"m"};
      my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

      if (! defined($host)) {
         return html(error("Could not create remote host object."));
      }

      if (! $host->isVMKernel()) {
         return html(error("Could not determine host memory configuration: " .
                           "The VMkernel is not loaded."));
      }
   }

   my %memInfo;

   # Server-wide total and available memory
   $memInfo{"total"} = $ctx->GetInt("/host2/#_mui/info/memory/sizeMb");
   $memInfo{"avail"} = $ctx->GetInt("/host2/#_mui/info/memory/sizeAvailMb");

   # Maximum new VM sizes (one setting per number of VCPUs)
   my @mxnew = ( 0 );
   foreach my $p (1 .. 2) {
      my $kbmax = $ctx->GetInt("/vmkernel/host/stats/memory/vmMaxSize/${p}p");
      push @mxnew, int($kbmax / 1024);
   }
   $memInfo{"mxnew"} = "[" . join(",", @mxnew) . "]";

   # Guest memory defaults for the current VM
   my $guest = $ctx->GetString("vmx/cfgState/val/guestOS");
   my $path = "/host2/#_mui/cfgLimits/guest/#/";
   while (defined($path = $ctx->GetNextSibling($path))) {
      if ($ctx->GetString("${path}key") =~ /\Q$guest\E/) {
         $memInfo{"memRec"} = $ctx->GetInt("${path}memory/defaultMb");
         $memInfo{"memMin"} = $ctx->GetInt("${path}memory/minMb");
      }
   }

   # Build the JS object and return
   my @memInfo = ();
   foreach (keys %memInfo) {
      push @memInfo, "$_:$memInfo{$_}";
   }

   return "var memInfo={" . join(",", @memInfo) . "};$/";
}


# ---------------------------------------------------------------------------

sub fsData {
   my $fs = shift();

   my $id    = $fs->getVmkName();
   my $vl    = $fs->getLogName();
   my $mb    = $fs->getCapacity();
   my $fmb   = $fs->getFreeCapacity();
   my $bs    = $fs->getBlockSize();
   my $ca    = $fs->getAccessibility();
   my @files = grep($_->[5] =~ /^disk$/i, @{$fs->listFiles()});

   my $fl = "";
   for (@files) {
      $fl .= ($fl ? ",$/" : "") . fileData($_);
   }
   $fl = "[$fl]";

   return qq{new FsVol("$id","$vl",$mb,$fmb,$bs,"$ca",$fl)};
}


# ---------------------------------------------------------------------------

sub fileData {
   my ($fn, $mb, $rwx, $uid, $gid, $att, $lmod) = @{shift()};

   return qq{new File("$fn",$mb,"$rwx",$uid,$gid,"$att","$lmod")};
}


# ---------------------------------------------------------------------------

sub fileExists {
   my $volume = shift();
   my $filename = shift();

   my %credentials = VMware::Management::Util::hash_credentials();
   my $user = $credentials{"l"};
   my $pass = $credentials{"m"};
   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

   if (! defined($host)) {
      my $msg .= error("Could not create remote host object.");
      return wantarray ? (0, $msg) : $msg;
   }

   if (! $host->isVMKernel()) {
      my $msg = error("Could not determine storage configuration: " .
                      "The VMkernel is not loaded.");
      return wantarray ? (0, $msg) : $msg;
   }

   my ($exists, $vmfs) = (0, undef);
   for my $fs (@{$host->getAllMFS()}) {
      if ($fs->getVmkName() eq $volume || $fs->getLogName() eq $volume) {
         $exists = scalar(grep(shift(@{$_}) eq $filename, @{$fs->listFiles()}));
         $vmfs = $fs;
      }
   }

   return wantarray ? (1, $exists, $vmfs) : $exists;
}


# ---------------------------------------------------------------------------

sub devData {
   my %dev = @_;
   my $str = "";

   for (keys(%dev)) {
      if ($dev{$_} =~ /^\d+$/ || $dev{$_} =~ /^(true|false|null)$/) {
         $str .= ($str ? "," : "") . qq{$_:$dev{$_}};
      } else {
         $str .= ($str ? "," : "") . qq{$_:"$dev{$_}"};
      }
   }

   return qq{{$str}};
}


# ---------------------------------------------------------------------------

sub floppy {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   # XXX: Defaults should come from VMDB.
   my $connected;  # Current connection state
   my $connectApo; # Connect at power on, or not
   my $hostif;     # Image file or host device
   my $loc;        # Of the file or device

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/floppy";

   $hostif = $ctx->GetString("$devPath/hostif");

   if ($hostif eq "device") {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/hostdev");
   } else {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/filePath");
   }

   $connected = isDevConnected($ctx, $dev);
   $connectApo = startConnected($ctx, $dev);

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/floppy";
      my $connect = 0;

      my @params = grep(/^\Q$dev\E/, $q->param());

      if ($connected && scalar(@params)) {
         $connect = 1;
      }

      for (@params) {
         if ("${dev}::" . &START_CONNECTED eq $_) {
            $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
         }

         if ("${dev}::" . &CONNECT_STATE eq $_) {
            $connect = $q->param($_);
         }

         if ("${dev}::" . &LOCATION eq $_) {
            # Device location is dependent on host interface.
            my $hi = $q->param("${dev}::" . &HOSTIF);
            if (defined($hi)) {
               $trnCtx->SetString("$setPath/hostif", $hi);
               $hostif = $trnCtx->GetString("$setPath/hostif");
            }

            if ($hostif eq "device") {
               $trnCtx->SetString("$setPath/hostif/$hostif/hostdev",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/hostdev");
            } else {
               $trnCtx->SetString("$setPath/hostif/$hostif/filePath",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/filePath");
            }
         }
      }

      $connected = touchRemDev($ctx, $dev, $trnCtx, $connect);
   }

   return ("id"             => $dev,
           &CONNECT_STATE   => $connected ? "true" : "false",
           &START_CONNECTED => $connectApo ? "true" : "false",
           &HOSTIF          => $hostif,
           &LOCATION        => jsStringNoQuote($loc));
}


# ---------------------------------------------------------------------------

sub scsiCtlr {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/scsiCtlr";

   my $emulation = $ctx->GetString("$devPath/virtualDev");
   my $sharedBus = $ctx->GetString("$devPath/sharedBus") || "none";

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/scsiCtlr";

      for ($q->param()) {
         VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
      }

      my @params = grep(/^\Q$dev\E/, $q->param());

      for (@params) {
         if ("${dev}::" . &EMULATION eq $_) {
            $trnCtx->SetString("$setPath/virtualDev", $q->param($_));
            $emulation = $trnCtx->GetString("$setPath/virtualDev");
            next;
         }

         if ("${dev}::" . &SHARED_BUS eq $_) {
            $trnCtx->SetString("$setPath/sharedBus", $q->param($_));
            $sharedBus = $trnCtx->GetString("$setPath/sharedBus") || "none";
            next;
         }
      }
   }

   return ("id"        => $dev,
           &SHARED_BUS => $sharedBus,
           &EMULATION  => $emulation);
}


# ---------------------------------------------------------------------------

sub getDevSubClass {
   my $ctx = shift();
   my $dev = shift();

   my $devNode = $dev; # VMDB dev node
   $devNode =~ s/^(scsiDev|ideDev).*/$1/;
   
   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/$devNode/subclass";
   return $ctx->GetString("$devPath");
}


# ---------------------------------------------------------------------------

sub storageDev {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   my %props;

   my $subclass;       # cdrom, disk or passthru
   my $hostif;         # device, image or file
   my $connected;      # Current connection state
   my $connectApo;     # Connect at power on, or not
   my $loc;            # Of the file or device
   my $mode;           # persistent, nonpersistent, undoable or append
   my $clientDevice; 
   my $newDev  = $dev; # In order to handle device ID changes
   my $devNode = $dev; # VMDB dev node
   $devNode =~ s/^(scsiDev|ideDev).*/$1/;

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/$devNode/subclass";
   $subclass = $ctx->GetString("$devPath");
   $devPath .= "/$subclass";

   # DVD/CD-ROM
   if ($subclass eq "cdrom") {
      $hostif = $ctx->GetString("$devPath/hostif");

      if ($hostif eq "device") {
         $loc = $ctx->GetString("$devPath/hostif/device/hostdev");
         $clientDevice = $ctx->GetString("$devPath/hostif/device/clientDevice");
      } else {
         $loc = $ctx->GetString("$devPath/hostif/image/filePath");
         $clientDevice = 0;
      }

      # Assume that cdrom devices are removable.
      $connected = isDevConnected($ctx, $dev);
      $connectApo = startConnected($ctx, $dev);

      $props{&CONNECT_STATE}   = $connected ? "true" : "false";
      $props{&START_CONNECTED} = $connectApo ? "true" : "false";
      $props{&CLIENT_DEVICE} = $clientDevice ? "true" : "false";

      if (defined($trnCtx)) {
         my $setPath = "in/val/dev/#_$dev/class/$devNode/subclass/$subclass";
         my $connect = 0;
         my $class = $dev;
         my $ctlrIx = $dev;
         my $ix = $dev;

         $dev =~ m/(scsiDev|ideDev)(\d):(\d)/i;
         $class = $1;
         $ctlrIx = $2;
         $ix = $3;

         for ($q->param()) {
            VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
         }

         my @params = grep(/^\Q$dev\E/, $q->param());

         if ($connected && scalar(@params)) {
            $connect = 1;
         }

         for (@params) {
            if ("${dev}::" . &START_CONNECTED eq $_) {
               $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
               next;
            }

            if ("${dev}::" . &CONNECT_STATE eq $_) {
               $connect = $q->param($_);
               next;
            }

            if ("${dev}::" . &LOCATION eq $_) {
               # Device location is dependent on host interface.
               my $hi = $q->param("${dev}::" . &HOSTIF);
               if (defined($hi)) {
                  $trnCtx->SetString("$setPath/hostif", $hi);
                  $hostif = $trnCtx->GetString("$setPath/hostif");
               }

               if ($hostif eq "device") {
                  $trnCtx->SetString("$setPath/hostif/$hostif/hostdev",
                                  $q->param($_));
                  $loc = $trnCtx->GetString("$setPath/hostif/$hostif/hostdev");

                  $clientDevice = $q->param("${dev}::" . &CLIENT_DEVICE);
                  if (defined($clientDevice)) {
                     $trnCtx->SetString("$setPath/hostif/$hostif/clientDevice",
                                        $clientDevice);
                     $clientDevice = $trnCtx->GetString("$setPath/hostif/$hostif/clientDevice");
                  }
               } else {
                  $trnCtx->SetString("$setPath/hostif/$hostif/filePath",
                                  $q->param($_));
                  $loc = $trnCtx->GetString("$setPath/hostif/$hostif/filePath");
                  $clientDevice = 0;
               }
               next;
            }

            # XXX: I don't know what the procedure for switching an IDE CD-ROM
            # to a SCSI Controller should be. I tried porting the code from
            # lib/vmuiVmdb/vmuiVmdb.c::VMUIVmdb_DevSetNode:
            #
            #      VMDB_ONERR(ret = VMUIVmdb_DevSetClass(ctx,dev,class), goto exit);
            #
            #      g_snprintf(buf,N,"%sclass/%s/ctlrIx/", dev, class);
            #      VMDB_ONERR(ret = Vmdb_SetInt(ctx,buf,node->major), goto exit);
            #
            #      g_snprintf(buf,N,"%sclass/%s/ix/", dev, class);
            #      VMDB_ONERR(ret = Vmdb_SetInt(ctx,buf,node->minor), goto exit);
            #
            #      g_snprintf(buf,N,"%sclass/%s/subclass/", dev, class);
            #      VMDB_ONERR(ret = Vmdb_Set(ctx,buf,subclass), goto exit);
            #
            # But that doesn't seem to work. I end up with a virtual disk.
            #
            # # A form element named ctrl will hose both Mozilla and MSIE. Why?
            # if ("${dev}::ctlr" eq $_) {
            #    my $wantClass = $q->param($_) . "Dev";
            #    my $wantCtlrIx = $q->param($q->param($_));
            #    $wantCtlrIx =~ s/^(\d):\d$/$1/;
            #    my $wantIx = $q->param($q->param($_));
            #    $wantIx =~ s/^\d:(\d)$/$1/;

            #    my $path = "vmx/cfgState/val/dev/#_$dev/class/";

            #    $ctx->SetString($path, $wantClass);
            #    $ctx->SetInt("${path}$class/ctlrIx", $wantCtlrIx);
            #    $ctx->SetInt("${path}$class/ix", $wantIx);
            #    $ctx->SetString("${path}$class/subclass", $subclass);
            # }
         }

         $connected = touchRemDev($ctx, $dev, $trnCtx, $connect);

         $props{&CONNECT_STATE}   = $connected ? "true" : "false";
         $props{&START_CONNECTED} = $connectApo ? "true" : "false";
         $props{&CLIENT_DEVICE} = $clientDevice ? "true" : "false";
      }
   }

   # Disk
   if ($subclass eq "disk") {
      $mode = $ctx->GetString("$devPath/mode");
      $hostif = $ctx->GetString("$devPath/hostif");
      $loc = $ctx->GetString("$devPath/file");

      if (defined($trnCtx)) {
         my $setPath = "in/val/dev/#_$dev/class/$devNode/subclass/$subclass";
         my $connect = 0;
         my $class = $dev;
         my $ctlrIx = $dev;
         my $ix = $dev;

         $dev =~ m/(scsiDev|ideDev)(\d):(\d)/i;
         $class = $1;
         $ctlrIx = $2;
         $ix = $3;
         for ($q->param()) {
            VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
         }

         my @params = grep(/^\Q$dev\E/, $q->param());

         if ($connected && scalar(@params)) {
            $connect = 1;
         }

         for (@params) {
            if ("${dev}::" . &LOCATION eq $_) {
               # Device location is dependent on host interface.
               my $hi = $q->param("${dev}::" . &HOSTIF);
               if (defined($hi)) {
                  $trnCtx->SetString("$setPath/hostif", $hi);
                  $hostif = $trnCtx->GetString("$setPath/hostif");
                  next;
               }

               if ($hostif eq "device") {
                  $trnCtx->SetString("$setPath/hostif/$hostif/hostdev",
                                  $q->param($_));
                  $loc = $trnCtx->GetString("$setPath/hostif/$hostif/hostdev");
               } else {
                  $trnCtx->SetString("$setPath/file",
                                  $q->param($_));
                  $loc = $trnCtx->GetString("$setPath/file");
               }
               next;
            }

            if ("${dev}::" . &MODE eq $_) {
               $trnCtx->SetString("$setPath/mode", $q->param($_));
               $mode = $trnCtx->GetString("$setPath/mode");
               next;
            }

            if (("${dev}::" . &SCSI_ID eq $_) && ($class eq "scsiDev")) {
               if ($q->param($_) =~ /^(\d):(\d)$/) {
                  if (checkAvailableIx($ctx, $dev, $1, $2)) {
                     my $classPath = "in/val/dev/#_$dev/class/$devNode";
                     $trnCtx->SetInt("$classPath/ctlrIx", $1);
                     $trnCtx->SetInt("$classPath/ix", $2);
                     $newDev = "scsiDev${1}:${2}";
                  } else {
                     VMware::Management::Util::trace("Tried to clobber SCSI ID '$_'", "error");
                  }
               } else {
                  VMware::Management::Util::trace("Invalid SCSI ID '$_'", "error");
               }
            }
            if (("${dev}::" . &IDE_ID eq $_) && ($class eq "ideDev")) {
               if ($q->param($_) =~ /^(\d):(\d)$/) {
                  if (checkAvailableIx($ctx, $dev, $1, $2)) {
                     my $classPath = "in/val/dev/#_$dev/class/$devNode";
                     $trnCtx->SetInt("$classPath/ctlrIx", $1);
                     $trnCtx->SetInt("$classPath/ix", $2);
                     $newDev = "ideDev${1}:${2}";
                  } else {
                     VMware::Management::Util::trace("Tried to clobber IDE ID '$_'", "error");
                  }
               } else {
                  VMware::Management::Util::trace("Invalid IDE ID '$_'", "error");
               }
            }
         }
      }

      $props{&MODE} = $mode;
      $props{&SIZE} = getDiskSize($ctx, $loc);
   }

   # Generic SCSI
   if ($subclass eq "passthru") {
      $loc = $ctx->GetString("$devPath/hostdev");

      # Assume that generic SCSI devices are removable.
      $connected = isDevConnected($ctx, $dev);
      $connectApo = startConnected($ctx, $dev);

      if (defined($trnCtx)) {
         my $setPath = "in/val/dev/#_$dev/class/$devNode/subclass/$subclass";
         my $connect = 0;
         my $class = $dev;
         my $ctlrIx = $dev;
         my $ix = $dev;

         $dev =~ m/(scsiDev)(\d):(\d)/i;
         $class = $1;
         $ctlrIx = $2;
         $ix = $3;

         for ($q->param()) {
            VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
         }

         my @params = grep(/^\Q$dev\E/, $q->param());

         if ($connected && scalar(@params)) {
            $connect = 1;
         }

         for (@params) {
            if ("${dev}::" . &START_CONNECTED eq $_) {
               $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
               next;
            }

            if ("${dev}::" . &CONNECT_STATE eq $_) {
               $connect = $q->param($_);
               next;
            }

            if ("${dev}::" . &LOCATION eq $_) {
               $trnCtx->SetString("$setPath/hostdev",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostdev");
               next;
            }

            if ("${dev}::" . &SCSI_ID eq $_) {
               if ($q->param($_) =~ /^(\d):(\d)$/) {
                  if (checkAvailableIx($ctx, $dev, $1, $2)) {
                     my $classPath = "in/val/dev/#_$dev/class/$devNode";
                     $trnCtx->SetInt("$classPath/ctlrIx", $1);
                     $trnCtx->SetInt("$classPath/ix", $2);
                     $newDev = "scsiDev${1}:${2}";
                  } else {
                     VMware::Management::Util::trace("Tried to clobber SCSI ID '$_'", "error");
                  }
               } else {
                  VMware::Management::Util::trace("Invalid SCSI ID '$_'", "error");
               }
            }
         }
      }

      $props{&CONNECT_STATE}   = $connected ? "true" : "false";
      $props{&START_CONNECTED} = $connectApo ? "true" : "false";
   }

   $props{"id"}      = $newDev;
   $props{&HOSTIF}   = "$subclass:$hostif";
   $props{&LOCATION} = jsStringNoQuote($loc);

   return %props;
}


# ---------------------------------------------------------------------------

sub getDiskSize {
   my $ctx = shift();
   my $loc = shift();

   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $loc = &VMware::Management::Util::getDiskPath($loc, $ctx->GetCurrentPath());
   if (!defined $loc) {
     return 0;
   }

   # get the disk size
   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $jobPath = $trnCtx->NewArrayIndex("/host2/#_mui/util/disk/cmd/##/");
   $jobPath = $trnCtx->GetAbsPath($jobPath);
   $trnCtx->SetString("${jobPath}op", "getDiskInfo");
   $trnCtx->SetString("${jobPath}user",
                      &VMware::Management::Session::get_usertoken());
   $jobPath = "${jobPath}op/getDiskInfo/";

   $trnCtx->SetString("${jobPath}in/file", $loc);

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
							     $jobPath);
   my $size = 0;
   if ($ok) {
      $size = $trnCtx->GetString("${jobPath}out/capacity/");
   }

   $jobPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($jobPath, 1);

   return $size;
}


# ---------------------------------------------------------------------------

sub memory {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   my $mb = $ctx->GetString("vmx/cfgState/val/dev/#_$dev/class/memory/sizeMb");
   # XXX: Hack. The client currently expects memory and vcpu info to be paired.
   my $num = $ctx->GetString("vmx/cfgState/val/dev/#_vcpu/class/vcpu/num");
   $num ||= 1; #not done for GSX

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/memory";

      for ($q->param()) {
         VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
      }

      my @params = grep(/^\Q$dev\E/, $q->param());

      for (@params) {
         if ("${dev}::" . &SIZE_MB eq $_) {
            $trnCtx->SetString("$setPath/sizeMb", $q->param($_));
            $mb = $trnCtx->GetString("$setPath/sizeMb");
            next;
         }

         if ("${dev}::" . &NUM eq $_) {
            $trnCtx->SetString("in/val/dev/#_vcpu/class/vcpu/num",
                            $q->param($_));
            $num = $trnCtx->GetString("in/val/dev/#_vcpu/class/vcpu/num");
            $num ||= 1; #not done for GSX
            next;
         }
      }
   }

   return ("id"     => $dev,
           &SIZE_MB => $mb,
           &NUM     => $num);
}


# ---------------------------------------------------------------------------

sub nic {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   my $connected;  # Current connection state
   my $connectApo; # Connect at power on, or not

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/nic";
   my $emulation = $ctx->GetString("$devPath/emulation");

   my $devName = $ctx->GetString("$devPath/devName");
   my $netName = $ctx->GetString("$devPath/networkName");
   my $connected = isDevConnected($ctx, $dev);
   my $connectApo = startConnected($ctx, $dev);

   my $hostif = ($netName ne "") ? "named" : $ctx->GetString("$devPath/hostif/");
   my $custom = "";
   if ($hostif eq "custom") {
      $custom = $ctx->GetString("$devPath/hostif/custom/vmnet");
   }

   $connected = isDevConnected($ctx, $dev);
   $connectApo = startConnected($ctx, $dev);

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/nic";
      my $connect = 0;

      my @params = grep(/^\Q$dev\E/, $q->param());

      if ($connected && scalar(@params)) {
         $connect = 1;
      }

      for (@params) {
         if ("${dev}::" . &START_CONNECTED eq $_) {
            $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
         }

         if ("${dev}::" . &CONNECT_STATE eq $_) {
            $connect = $q->param($_);
         }

         if ("${dev}::" . &DEV_NAME eq $_) {
            my %params = VMware::Management::Util::hashQueryString($q->param($_));
            
            $trnCtx->SetString("$setPath/devName", $params{&DEV_NAME});
            $devName = $trnCtx->GetString("$setPath/devName");

            $trnCtx->SetString("$setPath/networkName", $params{&NETWORK_NAME});
            $netName = $trnCtx->GetString("$setPath/networkName");
            next;
         }

         if ("${dev}::" . &EMULATION eq $_) {
            $trnCtx->SetString("$setPath/emulation", $q->param($_));
            $emulation = $trnCtx->GetString("$setPath/emulation");
            next;
         }

         if ("${dev}::" . &HOSTIF eq $_) {
            if ($q->param($_) eq "named") {
               # Network is named: assign name, don't set hostif
               my $nnm = $q->param("${dev}::" . &NETWORK_NAME);
               $trnCtx->SetString("$setPath/networkName", $nnm);
               $netName = $trnCtx->GetString("$setPath/networkName");
               $trnCtx->Unset("$setPath/hostif");
               $hostif = "named";
            } else {
               # Network is not named: set hostif, ignore name if provided
               $trnCtx->SetString("$setPath/hostif", $q->param($_));
               $hostif = $trnCtx->GetString("$setPath/hostif");
               $trnCtx->Unset("$setPath/networkName");

               if ($hostif eq "custom") {
                  my $ctm = $q->param("${dev}::" . &VMNET);
                  if (defined($ctm)) {
                     $trnCtx->SetString("$setPath/hostif/custom/vmnet", $ctm);
                     $custom = $trnCtx->GetString("$setPath/hostif/custom/vmnet");
                  }
               }
            }
            next;
         }
      }

      $connected = touchRemDev($ctx, $dev, $trnCtx, $connect);
   }

   return ("id"             => $dev,
           &HOSTIF          => $hostif,
           &VMNET           => $custom,
           &CONNECT_STATE   => $connected ? "true" : "false",
           &START_CONNECTED => $connectApo ? "true" : "false",
           &DEV_NAME        => $devName,
           &NETWORK_NAME    => $netName,
           &EMULATION       => $emulation);
}


# ---------------------------------------------------------------------------

sub video {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();

   my $depth = $ctx->GetString("vmx/cfgState/val/dev/#_$dev/class/video/depth");

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/video";

      for ($q->param()) {
         VMware::Management::Util::trace("ORIG: $_ = " . $q->param($_));
      }

      my @params = grep(/^\Q$dev\E/, $q->param());

      for (@params) {
         if ("${dev}::" . &DEPTH eq $_) {
            $trnCtx->SetString("$setPath/depth", $q->param($_));
            $depth = $trnCtx->GetString("$setPath/depth");
            next;
         }
      }
   }

   return ("id"   => $dev,
           &DEPTH => $depth);
}


# ---------------------------------------------------------------------------

sub serial {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();
   my %props;

   my $poll;       # Poll mode (1) or interrupt (0)
   my $hostif;     # device, file or pipe
   my $connected;  # Current connection state
   my $connectApo; # Connect at power on, or not
   my $endPoint;   # server or client
   my $endIsApp;   # tryNoRxLoss, or not
   my $loc;        # Of device, file or pipe

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/serial";

   $hostif = $ctx->GetString("$devPath/hostif/");

   if ($hostif eq "device") {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/hostdev");
   } else {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/filePath");
   }

   if ($hostif eq "pipe") {
      $endPoint = $ctx->GetString("$devPath/hostif/$hostif/endPoint");
      $endIsApp = $ctx->GetBool("$devPath/hostif/$hostif/tryNoRxLoss");

      $props{&END_POINT}      = $endPoint;
      $props{&TRY_NO_RX_LOSS} = $endIsApp ? "tnrlTrue" : "tnrlFalse";
   }

   $poll = $ctx->GetBool("$devPath/yieldCpuOnPoll");

   $connected = isDevConnected($ctx, $dev);
   $connectApo = startConnected($ctx, $dev);

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/serial";
      my $connect = 0;

      my @params = grep(/^\Q$dev\E/, $q->param());

      if ($connected && scalar(@params)) {
         $connect = 1;
      }

      for (@params) {
         if ("${dev}::" . &START_CONNECTED eq $_) {
            $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
         }

         if ("${dev}::" . &CONNECT_STATE eq $_) {
            $connect = $q->param($_);
         }

         if ("${dev}::" . &LOCATION eq $_) {
            # Device location is dependent on host interface.
            my $hi = $q->param("${dev}::" . &HOSTIF);
            if (defined($hi)) {
               $trnCtx->SetString("$setPath/hostif", $hi);
               $hostif = $trnCtx->GetString("$setPath/hostif");
            }

            if ($hostif eq "device") {
               $trnCtx->SetString("$setPath/hostif/$hostif/hostdev",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/hostdev");
            } else {
               $trnCtx->SetString("$setPath/hostif/$hostif/filePath",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/filePath");
            }

            if ($hostif eq "pipe") {
               $endPoint = $q->param("${dev}::" . &END_POINT);
               $trnCtx->SetString("$setPath/hostif/$hostif/endPoint", $endPoint);

               $endIsApp = $q->param("${dev}::" . &TRY_NO_RX_LOSS);
               $trnCtx->SetBool("$setPath/hostif/$hostif/tryNoRxLoss", 
                             ($endIsApp eq "tnrlTrue") ? 1 : 0);
               
               $endPoint = $trnCtx->GetString("$setPath/hostif/$hostif/endPoint");
               $endIsApp = $trnCtx->GetBool("$setPath/hostif/$hostif/tryNoRxLoss");

               $props{&END_POINT}      = $endPoint;
               $props{&TRY_NO_RX_LOSS} = $endIsApp ? "tnrlTrue" : "tnrlFalse";
            }
         }
         if ("${dev}::" . &YIELD_CPU_ON_POLL eq $_) {
            $poll = $q->param("${dev}::" . &YIELD_CPU_ON_POLL);
            $trnCtx->SetBool("$setPath/yieldCpuOnPoll", $poll);

            $poll = $trnCtx->GetBool("$setPath/yieldCpuOnPoll");
         }
      }

      $connected = touchRemDev($ctx, $dev, $trnCtx, $connect);
   }

   $props{"id"}               = $dev;
   $props{&HOSTIF}            = $hostif;
   $props{&CONNECT_STATE}     = $connected ? "true" : "false";
   $props{&START_CONNECTED}   = $connectApo ? "true" : "false";
   $props{&LOCATION}          = jsStringNoQuote($loc);
   $props{&YIELD_CPU_ON_POLL} = $poll ? "true" : "false";

   return %props;
}


# ---------------------------------------------------------------------------

sub parallel {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();
   my %props;

   my $bidi;       # Bidirectional, or not
   my $hostif;     # device or file
   my $connected;  # Current connection state
   my $connectApo; # Connect at power on, or not
   my $loc;        # Of device or file

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/parallel";

   $hostif = $ctx->GetString("$devPath/hostif/");

   if ($hostif eq "device") {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/hostdev");
      $bidi = $ctx->GetBool("$devPath/hostif/$hostif/bidirectional");

      $props{&BIDIRECTIONAL} = $bidi ? "true" : "false";
   } else {
      $loc = $ctx->GetString("$devPath/hostif/$hostif/filePath");
   }

   $connected = isDevConnected($ctx, $dev);
   $connectApo = startConnected($ctx, $dev);

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/parallel";
      my $connect = 0;

      my @params = grep(/^\Q$dev\E/, $q->param());

      if ($connected && scalar(@params)) {
         $connect = 1;
      }

      for (@params) {
         if ("${dev}::" . &START_CONNECTED eq $_) {
            $connectApo = startConnected($ctx, $dev, $trnCtx, $q->param($_));
         }

         if ("${dev}::" . &CONNECT_STATE eq $_) {
            $connect = $q->param($_);
         }

         if ("${dev}::" . &LOCATION eq $_) {
            # Device location is dependent on host interface.
            my $hi = $q->param("${dev}::" . &HOSTIF);
            if (defined($hi)) {
               $trnCtx->SetString("$setPath/hostif", $hi);
               $hostif = $trnCtx->GetString("$setPath/hostif");
            }

            if ($hostif eq "device") {
               $trnCtx->SetString("$setPath/hostif/$hostif/hostdev",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/hostdev");

               my $bi = $q->param("${dev}::" . &BIDIRECTIONAL);
               if (defined($bi)) {
                  $trnCtx->SetBool("$setPath/hostif/$hostif/bidirectional",
                                  $bi);
                  $bidi = $trnCtx->GetBool("$setPath/hostif/$hostif/bidirectional");
                  $props{&BIDIRECTIONAL} = $bidi ? "true" : "false";
               }
            } else {
               $trnCtx->SetString("$setPath/hostif/$hostif/filePath",
                               $q->param($_));
               $loc = $trnCtx->GetString("$setPath/hostif/$hostif/filePath");
            }
         }
      }

      $connected = touchRemDev($ctx, $dev, $trnCtx, $connect);
   }

   $props{"id"}             = $dev;
   $props{&HOSTIF}          = $hostif;
   $props{&CONNECT_STATE}   = $connected ? "true" : "false";
   $props{&START_CONNECTED} = $connectApo ? "true" : "false";
   $props{&LOCATION}        = jsStringNoQuote($loc);

   return %props;
}


# ---------------------------------------------------------------------------

sub usbCtlr {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();
   my %props;

   my $autoConnect;       # auto connect usb devices

   my $devPath = "vmx/cfgState/val/dev/#_$dev/class/usbCtlr";

   $autoConnect = $ctx->GetBool("$devPath/autoconnectDevices");

   $props{&AUTO_CONNECT} = $autoConnect;

   if (defined($trnCtx)) {
      my $setPath = "in/val/dev/#_$dev/class/usbCtlr";

      my @params = grep(/^\Q$dev\E/, $q->param());

      for (@params) {
         if ("${dev}::" . &AUTO_CONNECT eq $_) {
            $trnCtx->SetString("$setPath/autoconnectDevices",
                            $q->param($_));
            $autoConnect = $trnCtx->GetString("$setPath/autoconnectDevices");
         }
      }
   }

   $props{"id"}             = $dev;
   $props{&AUTO_CONNECT}    = $autoConnect;

   return %props;
}

# ---------------------------------------------------------------------------

sub isDevConnected {
   my $ctx = shift();
   my $dev = shift();

   if ($ctx->GetString("vmx/execState/val") !~ /^poweredon$/i) {
      return 0;
   }

   my $key = $ctx->GetString("vmx/cfgState/val/dev/#_$dev/key");

   my $path = "vmx/remDev/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      if ($key eq $ctx->GetString("${path}key")) {
         return $ctx->GetString("${path}connectState/val") =~ /^connected$/;
      }
   }

   return 0;
}


# ---------------------------------------------------------------------------

sub touchRemDev {
   VMware::Management::Util::trace("IN touchRemDev [", "debug");
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();
   my $val = shift();

   if ($ctx->GetString("vmx/execState/val") !~ /^poweredon$/i) {
      VMware::Management::Util::trace("] OUT touchRemDev", "debug");
      return 0;
   }

   my $key = $trnCtx->GetString("in/val/dev/#_$dev/key");
   VMware::Management::Util::trace("KEY: $key", "debug");

   my $reqPath = "in/remDev/#";
   my $found = 0;
   VMware::Management::Util::trace("EXAMINING: ${reqPath}", "debug");
   while (defined($reqPath = $ctx->GetNextSibling($reqPath))) {
      VMware::Management::Util::trace("EXAMINING: ${reqPath}key", "debug");
      if ($key eq $trnCtx->GetString("${reqPath}key")) {
	 VMware::Management::Util::trace("${reqPath}key", "debug");
         $found = 1;
         last;
      }
   }

   VMware::Management::Util::trace("FOUND: $found", "debug");

   if (! $found) {
      $reqPath = $trnCtx->NewArrayIndex("in/remDev/#");
      $trnCtx->SetString("${reqPath}key", $key);
   }

   $trnCtx->SetBool("${reqPath}connected", $val);

   VMware::Management::Util::trace("] OUT touchRemDev", "debug");
   return $trnCtx->GetBool("${reqPath}connected");
}


# ---------------------------------------------------------------------------

sub startConnected {
   my $ctx = shift();
   my $dev = shift();
   my $trnCtx = shift();
   my $val = shift();
   my $reqPath;

   if (defined($trnCtx)) {
      $reqPath = "in/val/dev/#_$dev/media/removable/startConnected";
      my $ret = $trnCtx->SetBool($reqPath, $val);

      VMware::Management::Util::trace("RET: $ret", "debug");

      return $trnCtx->GetBool($reqPath);
   }

   $reqPath = "vmx/cfgState/val/dev/#_$dev/media/removable/startConnected";
   return $ctx->GetBool($reqPath);
}


# ---------------------------------------------------------------------------

sub add {
   my $ctx = shift();
   my $data = "";

   if (! $q->param("dev")) {
      return html(error("Add Device: Device type not specified."));
   }

   my ($dev) = devices($ctx);

   if ($dev =~ /^serial$/i) {
      my ($ok, $msg) = addSerial($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^parallel$/i) {
      my ($ok, $msg) = addParallel($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^usbCtlr$/i) {
      my ($ok, $msg) = addUsbCtlr($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^floppy$/i) {
      my ($ok, $msg) = addFloppy($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^scsiCtlr$/i) {
      return html(error("Add Device: Not implemented."));
   }

   if ($dev =~ /^cdrom$/i) {
      my ($ok, $msg) = addDvdCdrom($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^disk$/i) {
      my ($ok, $msg) = addDisk($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^nic$/i) {
      my ($ok, $msg) = addNic($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   if ($dev =~ /^passthru$/i) {
      my ($ok, $msg) = addGenericScsi($ctx);
      $data .= $msg;
      return $ok ? data($ctx, $data) : html($data);
   }

   return html(error("Add Device: Unknown device type: $dev."));
}

sub addHead {
   my $ctx = shift();

   my ($trnCtx, $reqPath) = cmdHead($ctx);

   my $devPath = "${reqPath}in/val/dev/#";
   $devPath = $trnCtx->GetAbsPath($trnCtx->NewArrayIndex($devPath));

   return ($trnCtx, $reqPath, $devPath);
}

sub addTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = cmdTail($trnCtx, $reqPath);

   if (! $ok) {
      $data .= error("Add Device: $msg. Your changes were not saved.");
   }

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addFloppy {
   my $ctx = shift();
   my $dev = "floppy";
   my $data = "";

   my $ix = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add Floppy Drive: This virtual machine is already " .
                     "configured with the maximum number of floppy drives.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   my $connected  = 0;           # Current connection state
   my $connectApo = 1;           # Connect at power on, or not
   my $hostif     = "device";    # Image file or host device
   my $loc        = $^O eq "MSWin32" ? "A:" : "/dev/fd0";
                                  # File or device location

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);

   # Set defaults
   my $setPath = "${devPath}class/$dev/";
   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}hostif", $hostif);
   $trnCtx->SetString("${setPath}hostif/$hostif/hostdev", $loc);

   for ($q->param()) {
      if ("${dev}::" . &START_CONNECTED eq $_) {
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
                       $q->param($_));
      }

      if ("${dev}::" . &LOCATION eq $_) {
         # Device location is dependent on host interface.
         my $hi = $q->param("${dev}::" . &HOSTIF);
         if (defined($hi)) {
            $trnCtx->SetString("${setPath}hostif", $hi);
            $hostif = $trnCtx->GetString("${setPath}hostif");
         }

         if ($hostif eq "device") {
            $trnCtx->SetString("${setPath}hostif/$hostif/hostdev",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/hostdev");
         } else {
            $trnCtx->SetString("${setPath}hostif/$hostif/filePath",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/filePath");
         }
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addParallel {
   my $ctx = shift();
   my $dev = "parallel";
   my $data = "";

   my $ix = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add Parallel Port: This virtual machine is already " .
                     "configured with the maximum number of parallel port.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   my $connected  = 0;           # Current connection state
   my $connectApo = 1;           # Connect at power on, or not
   my $hostif     = "device";    # Image file or host device
   my $loc        = "/dev/parport0";  # Of the file or device
   my $bidi       = 0;           # Bidirectional, or not

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);

   # Set defaults
   my $setPath = "${devPath}class/$dev/";
   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}hostif", $hostif);
   $trnCtx->SetString("${setPath}hostif/$hostif/hostdev", $loc);

   for ($q->param()) {
      if ("${dev}::" . &START_CONNECTED eq $_) {
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
                       $q->param($_));
      }

      if ("${dev}::" . &LOCATION eq $_) {
         # Device location is dependent on host interface.
         my $hi = $q->param("${dev}::" . &HOSTIF);
         if (defined($hi)) {
            $trnCtx->SetString("${setPath}hostif", $hi);
            $hostif = $trnCtx->GetString("${setPath}hostif");
         }

         if ($hostif eq "device") {
            $trnCtx->SetString("${setPath}hostif/$hostif/hostdev",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/hostdev");

            my $bi = $q->param("${dev}::" . &BIDIRECTIONAL);
            if (defined($bi)) {
               $trnCtx->SetString("${setPath}hostif/$hostif/bidirectional",
                               $bi);
               $bidi = $trnCtx->GetString("${setPath}hostif/$hostif/bidirectional");
            }
         } else {
            $trnCtx->SetString("${setPath}hostif/$hostif/filePath",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/filePath");
         }
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addSerial {
   my $ctx = shift();
   my $dev = "serial";
   my $data = "";

   my $ix = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add Serial Port: This virtual machine is already " .
                     "configured with the maximum number of serial port.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   my $connected  = 0;           # Current connection state
   my $connectApo = 1;           # Connect at power on, or not
   my $hostif     = "device";    # Image file or host device
   my $loc        = "/dev/ttyS0";# Of the file or device
   my $endPoint   = "server";    # server or client
   my $endIsApp   = "tnrlFalse"; # tryNoRxLoss, or not
   my $poll       = 0;           # Poll mode (1) or interrupt (0)

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);

   # Set defaults
   my $setPath = "${devPath}class/$dev/";
   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}hostif", $hostif);
   $trnCtx->SetString("${setPath}hostif/$hostif/hostdev", $loc);

   for ($q->param()) {
      if ("${dev}::" . &START_CONNECTED eq $_) {
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
                       $q->param($_));
      }

      if ("${dev}::" . &LOCATION eq $_) {
         # Device location is dependent on host interface.
         my $hi = $q->param("${dev}::" . &HOSTIF);
         if (defined($hi)) {
            $trnCtx->SetString("${setPath}hostif", $hi);
            $hostif = $trnCtx->GetString("${setPath}hostif");
         }

         if ($hostif eq "device") {
            $trnCtx->SetString("${setPath}hostif/$hostif/hostdev",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/hostdev");
         } else {
            $trnCtx->SetString("${setPath}hostif/$hostif/filePath",
                            $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/filePath");
         }


         if ($hostif eq "pipe") {
            $endPoint = $q->param("${dev}::" . &END_POINT);
            if (defined($endPoint)) {
               $trnCtx->SetString("${setPath}hostif/$hostif/endPoint", $endPoint);
               $endPoint = $trnCtx->GetString("${setPath}hostif/$hostif/endPoint");
            }

            $endIsApp = $q->param("${dev}::" . &TRY_NO_RX_LOSS);
            if (defined($endIsApp)) {
               $trnCtx->SetBool("${setPath}hostif/$hostif/tryNoRxLoss", 
                             ($endIsApp eq "tnrlTrue") ? 1 : 0);
               $endIsApp = $trnCtx->GetBool("${setPath}hostif/$hostif/tryNoRxLoss");
            }
         }
      }

      if ("${dev}::" . &YIELD_CPU_ON_POLL eq $_) {
         $poll = $q->param("${dev}::" . &YIELD_CPU_ON_POLL);
         $trnCtx->SetBool("${setPath}yieldCpuOnPoll", $poll);
         $poll = $trnCtx->GetBool("${setPath}yieldCpuOnPoll");
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addUsbCtlr {
   my $ctx = shift();
   my $dev = "usbCtlr";
   my $data = "";

   my $ix = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add USB controller: This virtual machine is already " .
                     "configured with a USB controller.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   my $autoc = 0;           # auto connect devices

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);

   # Set defaults
   my $setPath = "${devPath}class/$dev/";
   $trnCtx->SetBool("${setPath}autoconnectDevices", $autoc);

   for ($q->param()) {
      if ("${dev}::" . &AUTO_CONNECT eq $_) {
         $trnCtx->SetBool("${setPath}autoconnectDevices", $q->param($_));
         $autoc = $trnCtx->GetBool("${setPath}autoconnectDevices");
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addDvdCdrom {
   my $ctx = shift();
   my $dev = "ideDev";
   my $subclass = "cdrom";
   my $data = "";

   my ($ctlrIx, $ix) = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add DVD/CD-ROM Drive: This virtual machine is already " .
                     "configured with the maximum number of IDE devices.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   my $connected  = 0;            # Current connection state
   my $connectApo = 1;            # Connect at power on, or not
   my $hostif     = "device";     # Image file or host device
   my $loc        = $^O eq "MSWin32" ? "auto detect" : "/dev/cdrom";
                                  # File or device location
   my $clientDevice = 0;          # client device

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ctlrIx", $ctlrIx);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);
   $trnCtx->SetString("${devPath}class/$dev/subclass", $subclass);

   # Set defaults
   my $setPath = "${devPath}class/$dev/subclass/$subclass/";

   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}hostif", $hostif);
   $trnCtx->SetString("${setPath}hostif/$hostif/hostdev", $loc);
   $trnCtx->SetBool("${setPath}hostif/$hostif/clientDevice", $clientDevice);

   for ($q->param()) {
      if ("${subclass}::" . &START_CONNECTED eq $_) {
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
                          $q->param($_));
      }

      if ("${subclass}::" . &LOCATION eq $_) {
         # Device location is dependent on host interface.
         my $hi = $q->param("${subclass}::" . &HOSTIF);
         if (defined($hi)) {
            $trnCtx->SetString("${setPath}hostif", $hi);
            $hostif = $trnCtx->GetString("${setPath}hostif");
         }

         if ($hostif eq "device") {
            $trnCtx->SetString("${setPath}hostif/$hostif/hostdev",
                               $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/hostdev");

            $clientDevice = $q->param("${subclass}::" . &CLIENT_DEVICE);
            if (defined($clientDevice)) {
               $trnCtx->SetString("${setPath}hostif/$hostif/clientDevice",
                               $clientDevice);
               $clientDevice = $trnCtx->GetString("${setPath}hostif/$hostif/clientDevice");
            }
         } else {
            $trnCtx->SetString("${setPath}hostif/$hostif/filePath",
                               $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/filePath");
         }
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ctlrIx:$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addGenericScsi {
   my $ctx = shift();
   my $dev = "scsiDev";
   my $subclass = "passthru";
   my $data = "";

   my ($ctlrIx, $ix);

   if ($q->param("${subclass}::" . &SCSI_ID) =~ /^(\d):(\d)$/) {
      # Check whether requested SCSI ID is in use
      ($ctlrIx, $ix) = ($1, $2);
      if (!checkAvailableIx($ctx, $dev, $ctlrIx, $ix)) {
         $data .= error("Add Generic SCSI Device: The requested Virtual " .
                        "SCSI Node (${ctlrIx}:${ix}) is already in use.");
         return wantarray ? (0, $data) : $data;
      }
   } else {
      # Find the first available SCSI ID
      ($ctlrIx, $ix) = getFirstAvailableIx($ctx, $dev);
      if (! defined($ix)) {
         $data .= error("Add Generic SCSI Device: This virtual machine is " .
                        "already configured with the maximum number of " .
                        "SCSI devices.");
         return wantarray ? (0, $data) : $data;
      }
   }

   # XXX: Defaults should come from VMDB.
   my $connected  = 0;            # Current connection state
   my $connectApo = 1;            # Connect at power on, or not
   my $loc        = ""; # Of the file or device

   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ctlrIx", $ctlrIx);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);
   $trnCtx->SetString("${devPath}class/$dev/subclass", $subclass);

   # Set defaults
   my $setPath = "${devPath}class/$dev/subclass/$subclass/";

   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}hostdev", $loc);

   for ($q->param()) {
      if ("${subclass}::" . &START_CONNECTED eq $_) {
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
                          $q->param($_));
      }

      if ("${subclass}::" . &LOCATION eq $_) {
         $trnCtx->SetString("${setPath}hostdev",
                            $q->param($_));
         $loc = $trnCtx->GetString("${setPath}hostdev");
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ctlrIx:$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addDisk {
   my $ctx = shift();
   my $dev = "scsiDev";
   my $subclass = "disk";
   my $data = "";
   my $fileExistsOk;
   my $exists = 0;
   my $vmfs;
   my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id");
   my $ctlr = $q->param("disk::ctlr");

   if ($ctlr eq "ide") {
      $dev = "ideDev";
   }
      
   if ($prdId eq "ESX") {
      # If we're not using a raw LUN, look for an existing disk file.
      if ((! defined($q->param("${subclass}::lun")) ||
           ! $q->param("${subclass}::lun"))) {
         ($fileExistsOk, $exists, $vmfs) = fileExists($q->param("disk::vl"),
                                                      $q->param("disk::fn"));

         if (! $fileExistsOk) {
            $data .= $exists; # An error message if not OK
            return wantarray ? (0, $data) : $data;
         } else {

            # Bail if the file exists and we were asked to create it.
            if ($exists && $q->param("disk::createDisk") == 1) {
               my $loc = $q->param("disk::vl") . ":" . $q->param("disk::fn");
               $data .= error(qq{Create Virtual Disk: $loc: } .
                              qq{Virtual disk already exists.});
               return wantarray ? (0, $data) : $data;
            }

            # Bail if the file doesn't exist and we were not asked to create it.
            if (! $exists && $q->param("disk::createDisk") == 0) {
               my $loc = $q->param("disk::vl") . ":" . $q->param("disk::fn");
               $data .= error(qq{Add Virtual Disk: $loc: } .
                              qq{No such virtual disk.});
               return wantarray ? (0, $data) : $data;
            }
         }
      }
   } else {
      my $loc = $q->param("disk::loc");

      # check the disk file location
      if ((!defined($loc) || !$loc)) {
         $data .= error(qq{Please specify the disk filename.});
         return wantarray ? (0, $data) : $data;
      }

      #check if the file exist, by checking it's disk size
      $exists = getDiskSize($ctx, $loc);
      
      # Bail if the file exists and we were asked to create it.
      if ($exists && $q->param("disk::createDisk") == 1) {
         $data .= error(qq{Create Virtual Disk: $loc: } .
                        qq{Virtual disk already exists.});
         return wantarray ? (0, $data) : $data;
      }

      # Bail if the file doesn't exist and we were not asked to create it.
      if (!$exists && $q->param("disk::createDisk") == 0) {
         $data .= error(qq{Add Virtual Disk: $loc: } .
                        qq{No such virtual disk.});
         return wantarray ? (0, $data) : $data;
      }
   }

   #
   # We're good to go. Try to add the disk as specified.
   #
   my ($ctlrIx, $ix);

   my $nodeId = $q->param("${subclass}::" . &SCSI_ID);
   if ($ctlr eq "ide") {
      $nodeId = $q->param("${subclass}::" . &IDE_ID);
   }

   if ($nodeId =~ /^(\d):(\d)$/) {
      # Check whether requested SCSI/IDE ID is in use
      ($ctlrIx, $ix) = ($1, $2);
      if (!checkAvailableIx($ctx, $dev, $ctlrIx, $ix)) {
         my $nodeType = ($dev eq "ideDev") ? "IDE" : "SCSI";
         $data .= error("Add Virtual Disk: The requested $nodeType ID " .
                        "(${ctlrIx}:${ix}) is already in use.");
         return wantarray ? (0, $data) : $data;
      }
   } else {
      # Find the first available SCSI/IDE ID
      ($ctlrIx, $ix) = getFirstAvailableIx($ctx, $dev);
      if (! defined($ix)) {
         my $nodeType = ($dev eq "ideDev") ? "IDE" : "SCSI";
         $data .= error("Add Virtual Disk: This virtual machine is already " .
                        "configured with the maximum number of $nodeType devices.");
         return wantarray ? (0, $data) : $data;
      }
   }


   #
   # Create the disk if we need to
   #
   if ($prdId eq "ESX") {
      # Let's see if we need to create a disk.
      if (! $exists && (! defined($q->param("${subclass}::lun")) ||
                        ! $q->param("${subclass}::lun"))) {
         # Bail if the specified VMFS volume doesn't exist.
         if (! defined($vmfs)) {
            $data .= error("Create Virtual Disk: " . $q->param("disk::vl") .
                           ": No such VMFS volume.");
            return wantarray ? (0, $data) : $data;
         }

         my $errStr = VMware::Management::SS::VMUtil::CreateVMFSFile($q->param("disk::vl"),
                                                                     $q->param("disk::fn"),
                                                                     $q->param("disk::mb"));

         # Bail if an error occurred while creating the disk.
         if ($errStr) {
            $data .= error("Add Virtual Disk: $errStr");
            return wantarray ? (0, $data) : $data;
         }
      }
   } else {
      if (!$exists && $q->param("disk::createDisk") == 1) {
        my $loc = $q->param("disk::loc");
        my $size = $q->param("disk::diskSize");
        my $flat = $q->param("disk::flat") ? 1 : 0;
        my $mono = $q->param("disk::split") ? 0 : 1;

        # check the disk capacity
        if ((!defined($size) || !$size)) {
           $data .= error(qq{Please specify the disk capacity.});
           return wantarray ? (0, $data) : $data;
        }

        $size = $size * 1024 * 1024 * 1024; # the diskSize is in G

         my $sep1 = "/";
         my $sep2 = "/";
         if ($^O eq "MSWin32") {
            $sep1 = "\\\\";
            $sep2 = "\\";
         }

         my $locPath;
         if ($loc =~ /^(.*)$sep1([^$sep1]*)$/) {
            $locPath = $loc;
         } else {
            my $defDir = $ctx->GetString("cfgPath");

            $defDir =~ /^(.*)$sep1([^$sep1]*)$/;
            $locPath = $1.$sep2.$loc;
         }

         # create the disk file
         my $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->SetCurrentPath($ctx->GetCurrentPath());
         $trnCtx->BeginTransaction();

         # Initialize VMDB request.
	 my $jobPath = $trnCtx->NewArrayIndex("/host2/#_mui/util/disk/cmd/##/");
	 $jobPath = $trnCtx->GetAbsPath($jobPath);
	 $trnCtx->SetString("${jobPath}op", "createDisk");
         $trnCtx->SetString("${jobPath}user",
                            &VMware::Management::Session::get_usertoken());
         $jobPath = "${jobPath}op/createDisk/";

         $jobPath = $trnCtx->GetAbsPath($jobPath);
         $trnCtx->SetString("${jobPath}in/file", $locPath);
         $trnCtx->SetString("${jobPath}in/createDir", "0");
         $trnCtx->SetString("${jobPath}in/ctlrType", $ctlr);
         $trnCtx->SetString("${jobPath}in/hostif", "file");
         $trnCtx->SetString("${jobPath}in/hostif/file/size", $size);
         $trnCtx->SetBool("${jobPath}in/hostif/file/flat", $flat);
         $trnCtx->SetBool("${jobPath}in/hostif/file/mono", $mono);
         $trnCtx->SetBool("${jobPath}in/progress", 1);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
								   $jobPath);

         $jobPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
         $trnCtx->LocalArrayUnset($jobPath, 1);

         if (!$ok) {
            $data .= error("Add Virtual Disk: $msg");
            return wantarray ? (0, $data) : $data;
         }
      }
   }


   # XXX: Defaults should come from VMDB.
   my $hostif = "file";       # Image file or host device
   my $mode   = "persistent"; # Of the file or device
   my $loc    = "";           # Of the file or device
  # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ctlrIx", $ctlrIx);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);
   $trnCtx->SetString("${devPath}class/$dev/subclass", $subclass);

   # Set defaults
   my $setPath = "${devPath}class/$dev/subclass/$subclass/";

   $trnCtx->SetString("${setPath}hostif", $hostif);
   if ($hostif eq "device") {
      $trnCtx->SetString("${setPath}hostif/$hostif/hostdev", $loc);
   } else {
      $trnCtx->SetString("${setPath}file", $loc);
   }
   $trnCtx->SetString("${setPath}mode", $mode);

   for ($q->param()) {
      if ("${subclass}::" . &MODE eq $_) {
         $trnCtx->SetString("${setPath}mode", $q->param($_));
         $mode = $trnCtx->GetString("${setPath}mode");
      }

      if ("${subclass}::" . &LOCATION eq $_) {
         # Device location is dependent on host interface.
         my $hi = $q->param("${subclass}::" . &HOSTIF);
         if (defined($hi)) {
            $trnCtx->SetString("${setPath}hostif", $hi);
            $hostif = $trnCtx->GetString("${setPath}hostif");
         }

         if ($hostif eq "device") {
            $trnCtx->SetString("${setPath}hostif/$hostif/hostdev",
                               $q->param($_));
            $loc = $trnCtx->GetString("${setPath}hostif/$hostif/hostdev");
         } else {
            $trnCtx->SetString("${setPath}file", $q->param($_));
            $loc = $trnCtx->GetString("${setPath}file");
         }
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ctlrIx:$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub addNic {
   my $ctx = shift();
   my $dev = "nic";
   my $data = "";
   my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id");

   my $ix = getFirstAvailableIx($ctx, $dev);

   if (! defined($ix)) {
      $data .= error("Add Network Adapater: This virtual machine is already " .
                     "configured with the maximum number of network " .
                     "adapters.");
      return wantarray ? (0, $data) : $data;
   }

   # XXX: Defaults should come from VMDB.
   # XXX: Defaults should come from VMDB.
   my $connected  = 0;            # Current connection state
   my $connectApo = 1;            # Connect at power on, or not
   my $devName   = "vmnic0";      # Which device to connect to
   my $netName   = "vmnic0";      # Which network to connect to
   my $emulation = "vlance";      # Device emulation (vlance or vmxnet)
   my $hostif    = "monitor_dev"; # Host interface (always monitor_dev for ESX)
   if ($prdId ne "ESX") {
      $emulation = "vmxnet";
      $hostif = "bridged";
   }
   my $custom;
   my $connected  = 0;           # Current connection state
   my $connectApo = 1;           # Connect at power on, or not
   
   # Begin transaction. Use $trnCtx for transaction operations.
   my ($trnCtx, $reqPath, $devPath) = addHead($ctx);

   # Set up device.
   $trnCtx->SetString("${devPath}class", $dev);
   $trnCtx->SetInt("${devPath}class/$dev/ix", $ix);

   # Set defaults
   my $setPath = "${devPath}class/$dev/";
   
   $trnCtx->SetString("${devPath}media", "removable");
   $trnCtx->SetBool("${devPath}media/removable/startConnected", $connectApo);
   $trnCtx->SetString("${setPath}devName", $devName);
   $trnCtx->SetString("${setPath}emulation", $emulation);
   $trnCtx->SetString("${setPath}hostif", $hostif);
   $trnCtx->SetString("${setPath}devName", $devName);

   for ($q->param()) {
      if ("${dev}::" . &DEV_NAME eq $_) {
         my %params = VMware::Management::Util::hashQueryString($q->param($_));

         $trnCtx->SetString("${setPath}devName", $params{&DEV_NAME});
         $devName = $trnCtx->GetString("${setPath}devName");

         $trnCtx->SetString("${setPath}networkName", $params{&NETWORK_NAME});
         $netName = $trnCtx->GetString("${setPath}networkName");
      }

      if ("${dev}::" . &EMULATION eq $_) {
         $trnCtx->SetString("${setPath}emulation", $q->param($_));
         $emulation = $trnCtx->GetString("${setPath}emulation");
      }

      if ("${dev}::" . &START_CONNECTED eq $_) {
	 VMware::Management::Util::trace("${devPath}", "debug");
         $trnCtx->SetBool("${devPath}media/removable/startConnected",
			  $q->param($_));
      }
      
      if ("${dev}::" . &HOSTIF eq $_) {

         if ($q->param($_) eq "named") {
            # Network is named: assign name, don't set hostif
            my $nnm = $q->param("${dev}::" . &NETWORK_NAME);
            $trnCtx->SetString("${setPath}networkName", $nnm);
            $netName = $trnCtx->GetString("${setPath}networkName");
            $trnCtx->Unset("${setPath}hostif");
            $hostif = "named";
         } else {
            # Network is not named: set hostif, ignore name if provided
            $trnCtx->SetString("${setPath}hostif", $q->param($_));
            $hostif = $trnCtx->GetString("${setPath}hostif");
            $trnCtx->Unset("${setPath}networkName");
            
            if ($hostif eq "custom") {
               my $ctm = $q->param("${dev}::" . &VMNET);
               if (defined($ctm)) {
                  $trnCtx->SetString("${setPath}hostif/custom/vmnet", $ctm);
                  $custom = $trnCtx->GetString("${setPath}hostif/custom/vmnet");
               }
            }
         }
      }
   }

   # End transaction.
   my ($ok, $msg) = addTail($trnCtx, $reqPath);
   $data .= $msg;

   $q->param("dev", "$dev$ix");

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub rem {
   my $ctx = shift();
   my $data = "";

   $ctx->BeginTransaction();

   my ($trnCtx, $reqPath) = cmdHead($ctx);

   # XXX: We should derive the array index for a device in a more robust manner.
   my $devPath = "${reqPath}in/val/dev/#_" . $q->param("dev") . "/";

   $trnCtx->SetInt("${devPath}present", 0);

   my ($ok, $msg) = cmdTail($trnCtx, $reqPath);
   if (! $ok) {
      $data .= error("Remove Device: $msg. Your changes were not saved.");
   }

   return html($data);
}


# ---------------------------------------------------------------------------

sub getFirstAvailableIx {
   my $ctx = shift();
   my $dev = shift();

   my $getPath = "vmx/cfgState/val/dev/#";

   if ($dev =~ /^(ide|scsi)Dev/) {
      my ($maxCtlr, $maxPerCtlr) = getMaxIxs($ctx, $dev);
      my @ctlrs;

      # Create an array of candidate ixs for each controller.
      #
      #      $ctlrs[0]->[0, 0, 0, 0, 0, 0, 0]
      #      $ctlrs[1]->[0, 0, 0, 0, 0, 0, 0]
      #
      for my $i (1..$maxCtlr) {
         my $ix = [];
         for my $x (1..$maxPerCtlr) {
            push(@{$ix}, 0);
         }
         push(@ctlrs, $ix);
      }

      # Figure out which controller ixs are filled.
      while (defined($getPath = $ctx->GetNextSibling($getPath))) {
         if ($ctx->GetString("${getPath}class") eq $dev) {
            my $ctlrIx = $ctx->GetString("${getPath}class/$dev/ctlrIx");
            my $ix = $ctx->GetString("${getPath}class/$dev/ix");
            $ctlrs[$ctlrIx]->[$ix] = $ctx->GetBool("${getPath}present");
         }
      }

      if ($dev =~ /^scsi/) {
         # For each controller...
         for my $ctlrIx (0..$#ctlrs) {
            # For each ix...
            for my $ix (0..$#{$ctlrs[$ctlrIx]}) {
               # If the ix is free...
               if (! $ctlrs[$ctlrIx]->[$ix]) {
                  return ($ctlrIx, $ix);
               }
            }
         }
      } else {
         # Put IDE devices on separate master controllers, if possible.
         # For each ix...
         for my $ix (0..$#{$ctlrs[0]}) {
            # For each controller...
            for my $ctlrIx (0..$#ctlrs) {
               # If the ix is free...
               if (! $ctlrs[$ctlrIx]->[$ix]) {
                  return ($ctlrIx, $ix);
               }
            }
         }
      }

      # There are no free controller ixs.
      return (undef, undef);
   }

   # Create an array of candidate ixs.
   my @ixs;
   for (1..getMaxIxs($ctx, $dev)) {
      push(@ixs, 0);
   }

   # Unsupported device
   if (! scalar(@ixs)) {
      return undef;
   }

   # Figure out which ixs are filled.
   while (defined($getPath = $ctx->GetNextSibling($getPath))) {
      if ($ctx->GetString("${getPath}class") eq $dev) {
         # $ixs[ix] = $present ? 1 : 0;
         $ixs[$ctx->GetString("${getPath}class/$dev/ix")] = $ctx->GetBool("${getPath}present");
      }
   }

   # For each ix...
   for my $ix (0..$#ixs) {
      # If the ix is free...
      if (! $ixs[$ix]) {
         return $ix;
      }
   }

   # There are no free ixs.
   return undef;
}

# ---------------------------------------------------------------------------

sub checkAvailableIx {
   my $ctx = shift();
   my $dev = shift();

   my $getPath = "vmx/cfgState/val/dev/#";

   if ($dev =~ /^(ide|scsi)Dev/) {
      my ($ctlrIx, $ix) = @_;
      while (defined($getPath = $ctx->GetNextSibling($getPath))) {
         if (($ctx->GetString("${getPath}class") eq $dev) &&
             ($ctlrIx == $ctx->GetString("${getPath}class/$dev/ctlrIx")) &&
             ($ix == $ctx->GetString("${getPath}class/$dev/ix")) &&
             ($ctx->GetBool("${getPath}present"))) {
            return 0;
         }
      }
   }
   return 1;
}


sub getMaxIxs {
   my $ctx = shift();
   my $dev = shift();

   my $lmtPath = "/host2/#_mui/cfgLimits/dev/";

   if ($dev =~ /^(audio|floppy|nic|parallel|serial|usbCtlr)/) {
      return $ctx->GetInt("${lmtPath}$1/max");
   }

   if ($dev =~ /^(scsi)Ctlr/) {
      my $max = $ctx->GetInt("${lmtPath}$1Dev/maxCtlr");
      return $max;
   }

   if ($dev =~ /^(ide|scsi)Dev/) {
      my $maxCtlr = $ctx->GetInt("${lmtPath}$1Dev/maxCtlr");
      my $maxPerCtlr = $ctx->GetInt("${lmtPath}$1Dev/maxPerCtlr");
      # Only support SCSI IDs 0-6 in the MUI
      if (($1 eq "scsi") && ($maxPerCtlr > 7)) {
         $maxPerCtlr = 7;
      }
      return ($maxCtlr, $maxPerCtlr);
   }

   return 0;
}


# ---------------------------------------------------------------------------

sub upgrade {
   my $ctx = shift();
   my $data = "";

   $ctx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $ctx->NewArrayIndex("vmx/upgradeJob/#");
   $reqPath = $ctx->GetAbsPath($reqPath);

   $ctx->SetBool("${reqPath}run", 1);

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndTransaction($ctx,
                                                                     $reqPath);

   if (! $ok) {
      $data .= error("Upgrade Virtual Hardware: $msg.");
   }

   $ctx->Unset($reqPath);

   # Invalidate any cached hwVersion value whether the upgrade succeeds or not,
   # just to be safe.
   for (devices($ctx)) {
      # Devices are in sort order. The first device on SCSI controller 0 is the
      # boot device. XXX: This will pick up 0:10 before 0:2 in the unlikely
      # event that A) someone configures the boot device on something other
      # than 0 or 1 -and- B) that same someone configures more than 9 devices
      # on a single SCSI controller (which isn't even possible via the MUI).
      if (/^scsiDev0:\d+$/i) {
         my $devPath = "vmx/cfgState/val/dev/#_$_/class/scsiDev/subclass/";
         my $subclass = $ctx->GetString($devPath);
         $devPath = "${devPath}$subclass/";

         if ($subclass eq "disk") {
            $ctx->SetInt("${devPath}hwVersion", -1);
            last;
         }
      }
   }

   return html($data);
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("cmd/##/");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", "setCfgState");
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());
   $reqPath .= "op/setCfgState/";
   VMware::VmdbPerl::CopyTree($ctx, $trnCtx, "vmx/cfgState/val/", "${reqPath}in/val/");

   return ($trnCtx, $reqPath);
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $msg) : $msg;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------
#
# jsStringNoQuote --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringNoQuote {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{$str};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}


# ---------------------------------------------------------------------------

sub updateHtSharing {
  my $ctx = shift();
  my $prdId = $ctx->GetString("/host2/#_mui/productinfo/id");

  my $cpuHtSupported = 0;
  my $cpuHtEnbl = 0;

  if ($prdId eq "ESX") {
    my %credentials = VMware::Management::Util::hash_credentials();
    my $user = $credentials{"l"};
    my $pass = $credentials{"m"};
    
    my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

    # HyperThreading supported ?
    $cpuHtSupported = $host->isCPUHyperthreadingEnabled();

    # configDB is only a ESX config
    # HyperThreading enabled/disabled ?
    my $cfgDb = $host->getConfigDB();
    $cpuHtEnbl = $cfgDb->get("hyperthreading");
  }

  if ( $cpuHtSupported != 0 ) { 
    my $vmid = $q->param("vmid");

    my $numVCpu = $ctx->GetInt("/vm/#".$vmid."/vmx/cfgState/val/dev/#_vcpu/class/vcpu/num");

    # Create a new context for this transaction to avoid 
    # VMDB_E_DISTRIBUTED_COMMIT errors.
    my $trnCtx = &VMware::Management::Session::get_temp_context();
    $trnCtx->SetCurrentPath($ctx->GetCurrentPath());
    my $reqPath = "/vmkernel/resources/vm/#".$vmid."/";

    # find out what the current status of htSharing is for the VM
    my $cpuHtCurrent = $trnCtx->GetString("${reqPath}val/cpu/htSharing/");

    # need to change htsharing value only if VM hyper-threading is isolated
    if ( $cpuHtCurrent ne "any" ) { 
      
      if ( ($numVCpu > 1) && ($cpuHtCurrent eq "none") ||
	   ($numVCpu == 1) && ($cpuHtCurrent eq "internal")
	) { 

	$trnCtx->BeginTransaction();
	my $reqPath = $trnCtx->NewArrayIndex("${reqPath}cmd/##");
	$reqPath = $trnCtx->GetAbsPath($reqPath);
	$trnCtx->SetString("${reqPath}op", "setResources");
	$trnCtx->SetString("${reqPath}user",
                           &VMware::Management::Session::get_usertoken());

	if ( ($numVCpu > 1) && ($cpuHtCurrent eq "none") ) {
	  $trnCtx->SetString("${reqPath}op/setResources/in/cpu/htSharing/", "internal");
	} else {
	  $trnCtx->SetString("${reqPath}op/setResources/in/cpu/htSharing/", "none");
	}
	
	my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd(
				 $trnCtx, "${reqPath}op/setResources/");

	if ( ! $ok ) {
	  VMware::Management::Util::trace(
	     "Problem setting the ht status ! returned : $msg","debug");
	}
	
        $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
        $trnCtx->LocalArrayUnset($reqPath, 1);
      }
    }
  }
}

1;
