# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::VmOptions;
# File: VMware/Management/VmOptions.pm

use strict;

use VMware::Management::Util ();
use VMware::VmdbPerl ();
use VMware::Management::VmdbUtil ();
use VMware::Management::Session ();

use CGI ();
my $q;

# Boolean options
my %BPATH = ("enableLogging"    => "val/opt/enableLogging/",
             "runWithDebugInfo" => "val/opt/runWithDebugInfo/",
	     "autostartContinueWhenToolsRun" => 
	                 "val/autostartContinueWhenToolsRun/");

# String options
#"val/opt/suspendDirectory isn't working on GSX, why !!!!
# the UI is using redoLogDir, I will do the same --Patrick
my %SPATH = ("suspendDirectoryGSX" => "val/opt/redoLogDir/",
             "suspendDirectory" => "val/opt/suspendDirectory/",
             "guestOS"          => "val/guestOS/",
             "name"             => "val/name/",
	     "autostart"        => "val/autostart/",
	     "autostop"         => "val/autostop/");

# Integer options
my %IPATH = ("autostartOrder"   => "val/autostartOrder/",
	     "autostopOrder"    => "val/autostopOrder/",
	     "autostartDelay"   => "val/autostartDelay/",
	     "autostopDelay"    => "val/autostopDelay/");

# Raw Options
my $RPATH = "val/all/";


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $ctx = &VMware::Management::Session::get_temp_context();
   $ctx->SetCurrentPath("/");

   for ($q->param()) {
      VMware::Management::Util::trace("OPT $_: " . $q->param($_));
   }

   my $vmid = $q->param("vmid");
   if (defined($vmid) && $vmid ne "") {
      if (isRegisteredVmId($ctx, $vmid)) {
         $ctx->SetCurrentPath("/vm/#$vmid/");
      } else {
         return html(error("Virtual machine not found: #$vmid."));
      }
   } else {
      return html(error("Missing virtual machine ID argument."));
   }

   if ($q->param("op") =~ /^(set)$/i) {
      return set($ctx);
   }

   return data($ctx);
}


# ---------------------------------------------------------------------------

sub isRegisteredVmId {
   my $ctx = shift();
   my $vmid = shift();

   my $path = "/host2/#_mui/vms/registeredvms/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      if ($ctx->GetString("${path}vm") =~ /#\Q$vmid\E/) {
         return 1;
      }
   }

   return 0;
}


# ---------------------------------------------------------------------------

sub options {
   my $ctx = shift();
   my $raw = caller() eq __PACKAGE__ ? $q->param("raw") : 0;
   my $opt = caller() eq __PACKAGE__ ? $q->param("opt") : undef;
   my @options;

   if (defined($opt)) {
      @options = split(",", $opt);
   } elsif (defined($raw) && $raw) {
      $ctx->SetCurrentPath("vmx/rawCfgState/");
      my $vmid = $q->param("vmid");
      my $path = $ctx->GetCurrentPath() . "${RPATH}#";
      VMware::Management::Util::trace("RAW PATH: $RPATH", "debug");
      while (defined($path = $ctx->GetNextSibling($path))) {
         push(@options, $ctx->GetString("${path}key"));
         push(@options, $ctx->GetString("${path}value"));
      }
      $ctx->SetCurrentPath("../../");
   } else {
      @options = ("enableLogging",
                  "guestOS",
                  "name",
                  "runWithDebugInfo",
                  "suspendDirectoryGSX",
                  "suspendDirectory",
		  "autostartContinueWhenToolsRun",
		  "autostart",
		  "autostop",
		  "autostartOrder",
		  "autostopOrder",
		  "autostartDelay",
		  "autostopDelay");
   }

   return @options;
}


# ---------------------------------------------------------------------------

sub data {
   my $ctx = shift();
   my $data = shift() || "";
   my $raw = $q->param("raw");
   my $optData = "";

   if (! defined($raw) || ! $raw) {
      for (options($ctx)) {
         $optData .= ($optData ? ",$/" : "") . optData($ctx, $_);
      }
   } else {
      my %options = options($ctx);
      for (sort {uc($a) cmp uc($b)} keys %options) {
         $optData .= ($optData ? ",$/" : "") . optData($ctx, $_, $options{$_});
      }
   }

   $data .= qq{var opts=[$optData];};

   VMware::Management::Util::trace("DATA: $data", "debug");
   return html($data);
}


# ---------------------------------------------------------------------------

sub optData {
   my $ctx = shift();
   my $opt = shift();
   my $val = shift();
   my $raw = caller() eq __PACKAGE__ ? $q->param("raw") : 0;

   if (! defined($raw) || ! $raw) {
      $ctx->SetCurrentPath("vmx/cfgState/");
      if (grep($_ eq $opt, keys(%BPATH))) {
         # Boolean Options
         $val = $ctx->GetBool($BPATH{$opt}) ? "true" : "false";
      } elsif (grep($_ eq $opt, keys(%IPATH))) {
         # Integer options
         $val = $ctx->GetInt($IPATH{$opt});
      } else {
         # Default is string
         $val = jsStringLiteral($ctx->GetString($SPATH{$opt}));
      }
      $ctx->SetCurrentPath("../../");
   } else {
      $val = defined($val) ? jsStringLiteral($val) : qq{""};
   }

   return qq{{k:"$opt",v:$val}};
}


# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $op = $q->param("raw") ? "setRawCfgState" : "setCfgState";
   my $reqPath = $trnCtx->NewArrayIndex("cmd/##/");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", $op);
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());
   $reqPath .= "op/$op/";
   my $state = $q->param("raw") ? "rawCfgState" : "cfgState";
   VMware::VmdbPerl::CopyTree($ctx, $trnCtx, "vmx/$state/val/", "${reqPath}in/val/");

   return ($trnCtx, $reqPath);
}


# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
      $data .= error("Edit Options: $msg. Your changes were not saved.");
   }

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub set {
   my $ctx = shift();
   my $data = "";
   my $raw = $q->param("raw");
   my ($ok, $trnCtx, $reqPath);

   ($trnCtx, $reqPath) = cmdHead($ctx);

   # translate autostart option, if set
   my $autostart = $q->param("autostart");
   if ($autostart eq "0") {
     $q->param("autostart", "none");
     $q->param("autostartOrder", 0);
     $q->param("autostopOrder", 0);
   } elsif ($autostart eq "1") {
     $q->param("autostart", "poweron");
   }

   if (! defined($raw) || ! $raw) {
     for my $opt (options($trnCtx)) {
       if (grep($_ eq $opt, keys(%BPATH)) && defined($q->param($opt))) {
	 # Boolean Options
	 $trnCtx->SetBool($reqPath . "in/" . $BPATH{$opt}, $q->param($opt));
       } elsif (grep($_ eq $opt, keys(%IPATH)) && defined($q->param($opt))) {
	 # Integer options
	 $trnCtx->SetInt($reqPath . "in/" . $IPATH{$opt}, $q->param($opt));
       } elsif (defined($q->param($opt))) {
	 # Default is string
	 $trnCtx->SetString($reqPath . "in/" . $SPATH{$opt}, $q->param($opt) || "");
       }
     }
   } else {
      for my $opt ($q->param()) {
         # Ignore CGI params that are not config file options.
         if ($opt =~ /^(vmid|op|opt|raw)$/i) {
            next;
         }

         my $val = $q->param($opt);
         my $ret = $trnCtx->SetString("${reqPath}in/${RPATH}#_${opt}/value", $val);
         my $ret = $trnCtx->SetString("${reqPath}in/${RPATH}#_${opt}/key", $opt);
         if ($ret != 0) {
            my $msg = VMware::Management::VmdbUtil::vmdbMessageForCode($ret);
            $data .= warning($msg);
         }
      }
   }

   ($ok, $data) = cmdTail($trnCtx, $reqPath);

   return data($ctx, $data);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   VMware::Management::Util::trace($msg, "error");
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   VMware::Management::Util::trace($msg, "warning");
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
