# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::VmdbUtil;
# File: VMware/Management/VmdbUtil.pm

use VMware::VmdbPerl ();

my $VMDB_S_MASTER_CHANGED         =  3;   # The master of a snapshot changed
my $VMDB_S_NOT_FOUND              =  2;
my $VMDB_S_FALSE                  =  1;
my $VMDB_S_OK                     =  0;
my $VMDB_E_FAIL                   = -1;
my $VMDB_E_NOT_FOUND              = -2;
my $VMDB_E_NOT_IMPL               = -3;
my $VMDB_E_ROOT                   = -4;   # illegal operation on the root
my $VMDB_E_ALREADY_EXISTS         = -5;
my $VMDB_E_INVALID_ARG            = -6;
my $VMDB_E_MEMORY                 = -7;
my $VMDB_E_ALREADY_SUBSCRIBED     = -8;
my $VMDB_E_POINTER                = -9;
my $VMDB_E_INVALID_TYPE           = -10;
my $VMDB_E_TYPE_MISMATCH          = -11;
my $VMDB_E_MASTER_CHANGED         = -12;  # The master of snapshot changed
my $VMDB_E_SCHEMA_MISMATCH        = -13;
my $VMDB_E_PIPE                   = -14;  # Broken pipe
my $VMDB_E_PATH_NOT_LOCAL         = -15;
my $VMDB_E_INVALID_PATH           = -16;
my $VMDB_E_NESTED_XREF            = -17;  # The xref schema embeds an xref
my $VMDB_E_NULL_XREF              = -18;  # The xref value has not been set
my $VMDB_E_ONEOF                  = -19;  # A one-of constraint was violated
my $VMDB_E_STALE_DATA             = -20;  # The transaction touched stale data
my $VMDB_E_ALREADY_IN_TRANSACTION = -21;
my $VMDB_E_CIRCULAR_XREF          = -22;
my $VMDB_E_BUFFER                 = -23;  # The supplied buffer is too small
my $VMDB_E_NOT_IN_TRANSACTION     = -24;
my $VMDB_E_PIPE_WOULD_BLOCK       = -26;  # The pipe operation would block
my $VMDB_E_PIPE_EOM               = -27;  # Encounted EOM
my $VMDB_E_PIPE_BAD_VERSION       = -28;
my $VMDB_E_PIPE_BAD_TOKEN         = -29;  # Received a bad string token
my $VMDB_E_DISTRIBUTED_COMMIT     = -30;  # Can't commit accross remote DBs
my $VMDB_E_PIPE_WRITE_FAILED      = -31;  # Relevant to IVmdbPipe->Flush
my $VMDB_E_PIPE_READ_FAILED       = -32;  # Relevant to IVmdbPipe->ProcessIO
my $VMBD_E_PIPE_READ_ERROR        = -33;  # Relevant to IVmdbPipe->ReadStr
my $VMDB_E_NULL_POLL              = -34;  # Unspecified poll registrar
my $VMDB_E_NULL                   = -35;  # The underlying object is NULL (lang. bindings)
my $VMDB_E_NOT_NULL               = -36;  # The underlying object is not NULL
my $VMDB_E_NO_PERMISSION          = -37;  # No permissions to perform the operation
my $VMDB_E_INVALID_SUBJ_SID       = -38;
my $VMDB_E_INVALID_OBJ_SID        = -39;
my $VMDB_E_CALLBACK               = -40;  # A needed callback function was not provided
my $VMDB_E_TIMEOUT                = -41;
my $VMDB_E_FORMAT                 = -42;  # the provided shared memory was not formatted
my $VMDB_E_BAD_STATE              = -43;  # Operation inconsistent with current state
my $VMDB_E_MSG                    = -44;  # Error information via Msg_GetMessagesAndReset()
my $VMDB_E_CANNOT_CONNECT_TO_VMX  = -45;
my $VMDB_E_VMX_DIED               = -46;

my %DEFAULT_MSG;

$DEFAULT_MSG{$VMDB_S_MASTER_CHANGED}         = "Master changed";
$DEFAULT_MSG{$VMDB_S_NOT_FOUND}              = "Not found";
$DEFAULT_MSG{$VMDB_S_FALSE}                  = "False";
$DEFAULT_MSG{$VMDB_S_OK}                     = "OK";
$DEFAULT_MSG{$VMDB_E_FAIL}                   = "Fail";
$DEFAULT_MSG{$VMDB_E_NOT_FOUND}              = "Not found";
$DEFAULT_MSG{$VMDB_E_NOT_IMPL}               = "Not implemented";
$DEFAULT_MSG{$VMDB_E_ROOT}                   = "Root";
$DEFAULT_MSG{$VMDB_E_ALREADY_EXISTS}         = "File or directory already exists";
$DEFAULT_MSG{$VMDB_E_INVALID_ARG}            = "Invalid argument";
$DEFAULT_MSG{$VMDB_E_MEMORY}                 = "Memory";
$DEFAULT_MSG{$VMDB_E_ALREADY_SUBSCRIBED}     = "Already subscribed";
$DEFAULT_MSG{$VMDB_E_POINTER}                = "Pointer";
$DEFAULT_MSG{$VMDB_E_INVALID_TYPE}           = "Invalid type";
$DEFAULT_MSG{$VMDB_E_TYPE_MISMATCH}          = "Type mismatch";
$DEFAULT_MSG{$VMDB_E_MASTER_CHANGED}         = "Master changed";
$DEFAULT_MSG{$VMDB_E_SCHEMA_MISMATCH}        = "Schema mismatch";
$DEFAULT_MSG{$VMDB_E_PIPE}                   = "Pipe";
$DEFAULT_MSG{$VMDB_E_PATH_NOT_LOCAL}         = "Path not local";
$DEFAULT_MSG{$VMDB_E_INVALID_PATH}           = "Invalid path";
$DEFAULT_MSG{$VMDB_E_NESTED_XREF}            = "Nested cross reference";
$DEFAULT_MSG{$VMDB_E_NULL_XREF}              = "Null cross reference";
$DEFAULT_MSG{$VMDB_E_ONEOF}                  = "One of";
$DEFAULT_MSG{$VMDB_E_STALE_DATA}             = "Stale data";
$DEFAULT_MSG{$VMDB_E_ALREADY_IN_TRANSACTION} = "Already in transaction";
$DEFAULT_MSG{$VMDB_E_CIRCULAR_XREF}          = "Circular cross reference";
$DEFAULT_MSG{$VMDB_E_BUFFER}                 = "Buffer";
$DEFAULT_MSG{$VMDB_E_NOT_IN_TRANSACTION}     = "not in transaction";
$DEFAULT_MSG{$VMDB_E_PIPE_WOULD_BLOCK}       = "Pipe: Would block";
$DEFAULT_MSG{$VMDB_E_PIPE_EOM}               = "Pipe: End of message";
$DEFAULT_MSG{$VMDB_E_PIPE_BAD_VERSION}       = "Pipe: Bad version";
$DEFAULT_MSG{$VMDB_E_PIPE_BAD_TOKEN}         = "Pipe: Bad token";
$DEFAULT_MSG{$VMDB_E_DISTRIBUTED_COMMIT}     = "Cannot commit accross distributed paths";
$DEFAULT_MSG{$VMDB_E_PIPE_WRITE_FAILED}      = "Pipe: Write failed";
$DEFAULT_MSG{$VMDB_E_PIPE_READ_FAILED}       = "Pipe: Read failed";
$DEFAULT_MSG{$VMBD_E_PIPE_READ_ERROR}        = "Pipe: Read error";
$DEFAULT_MSG{$VMDB_E_NULL_POLL}              = "Null poll";
$DEFAULT_MSG{$VMDB_E_NULL}                   = "Null";
$DEFAULT_MSG{$VMDB_E_NOT_NULL}               = "Not null";
$DEFAULT_MSG{$VMDB_E_NO_PERMISSION}          = "Permission denied";
$DEFAULT_MSG{$VMDB_E_INVALID_SUBJ_SID}       = "Invalid subject SID";
$DEFAULT_MSG{$VMDB_E_INVALID_OBJ_SID}        = "Invalid object SID";
$DEFAULT_MSG{$VMDB_E_CALLBACK}               = "Callback";
$DEFAULT_MSG{$VMDB_E_TIMEOUT}                = "Operation timed out";
$DEFAULT_MSG{$VMDB_E_FORMAT}                 = "Format";
$DEFAULT_MSG{$VMDB_E_BAD_STATE}              = "Bad state";
$DEFAULT_MSG{$VMDB_E_MSG}                    = "Message";
$DEFAULT_MSG{$VMDB_E_CANNOT_CONNECT_TO_VMX}  = "Cannot connect to VMX";
$DEFAULT_MSG{$VMDB_E_VMX_DIED}               = "VMX died";

sub vmdbMessageForCode {
   return $DEFAULT_MSG{shift()} || "Unknown error";
}

sub vmdbEndTransaction {
   my $ctx = shift();
   my $reqPath = shift();
   my $errPath = shift() || "status/error/text/";
   my $timeout = shift() || 30;
   my $ok = 1;
   my $msg = "";

   my $ret = $ctx->EndTransaction(1);

   if ($ret != 0) {
      $ok = 0;
      $msg = VMware::Management::VmdbUtil::vmdbMessageForCode($ret);
   } else {
      # Sleep in 10ms intervals
      my $sleep = 0.01;
      my $limit = $timeout / $sleep;
      my $percent = 0;
      my $count = 0;

      while ($count < $limit) {
         my $status = $ctx->GetString("${reqPath}status");
         if ($status eq "done") {
            last;
         } elsif ($status eq "error") {
            ($ok, $msg) = (0, $ctx->GetString("${reqPath}${errPath}"));
            last;
         } elsif ($count == $limit) {
            ($ok, $msg) = (0, "Operation timed out");
            last;
         } elsif ($status eq "inProgress") {
            my $newPercent = $ctx->GetInt("${reqPath}status/inProgress/percentDone");
            if ($newPercent > $percent) {
               # We're making progress -- reset the timeout counter
               $percent = $newPercent;
               $count = 0;
            }
         }
         select(undef, undef, undef, $sleep);
         $count++;
      }
   }

   return wantarray ? ($ok, $msg) : $msg;
}


sub vmdbEndCmd {
   my ($ctx, $cmdPath, $timeout) = @_;

   $cmdPath =~ s{^(.*)(cmd/\#\#.*?/).*}{$1$2};

   return vmdbEndTransaction($ctx, $cmdPath, "status/error/errMsg/", $timeout);
}
